@@ -1,4 +1,4 @@
-from typing import List, Tuple
+from typing import List, Tuple, Union
 import pandas as pd
 from ..wrapper.mysql import RawDatabaseConnector
 from ..view.raw_models import *
@@ -19,7 +19,7 @@ class RawDataApi:
                 return df
 
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {CmIndexPrice.__tablename__}')
 
     def get_cxindex_index_price_df(self, start_date, end_date):
         with RawDatabaseConnector().managed_session() as db_session:
@@ -34,7 +34,7 @@ class RawDataApi:
                 return df
 
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {CxindexIndexPrice.__tablename__}')
 
     def get_yahoo_index_price_df(self, start_date, end_date):
         with RawDatabaseConnector().managed_session() as db_session:
@@ -49,7 +49,7 @@ class RawDataApi:
                 return df
 
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {YahooIndexPrice.__tablename__}')
 
     def get_rq_index_price_df(self, start_date, end_date):
         with RawDatabaseConnector().managed_session() as db_session:
@@ -80,7 +80,7 @@ class RawDataApi:
                 return df
 
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindFundInfo.__tablename__}')
 
     def get_fund_fee(self):
         with RawDatabaseConnector().managed_session() as db_session:
@@ -142,17 +142,22 @@ class RawDataApi:
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
-    def get_rq_index_weight(self, index_id_list, start_date):
+    def get_rq_index_weight(self, index_id_list, start_date: str, end_date: str = ''):
         with RawDatabaseConnector().managed_session() as quant_session:
             try:
                 query = quant_session.query(
                         RqIndexWeight.index_id,
                         RqIndexWeight.datetime,
                         RqIndexWeight.stock_list,
+                        RqIndexWeight.weight_list,
                     ).filter(
                         RqIndexWeight.index_id.in_(index_id_list),
                         RqIndexWeight.datetime >= start_date,
                     )
+                if end_date:
+                    query = query.filter(
+                        RqIndexWeight.datetime <= end_date,
+                    )
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
 
@@ -217,7 +222,7 @@ class RawDataApi:
                     )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmTradeDates.__tablename__}')
                 return None
 
     def get_stock_info(self):
@@ -254,12 +259,12 @@ class RawDataApi:
                     EmFundNav
                 ).filter(
                     EmFundNav.DATES >= start_date,
-                    EmFundNav.DATES <= end_date
+                    EmFundNav.DATES <= end_date,
                 )
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmFundNav.__tablename__}')
                 return None
 
     def delete_em_fund_nav(self, start_date, end_date):
@@ -274,7 +279,7 @@ class RawDataApi:
                 db_session.commit()
                 return True
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to delete data <err_msg> {e} from {EmFundNav.__tablename__}')
                 return None
 
     def get_rq_fund_size(self):
@@ -338,7 +343,7 @@ class RawDataApi:
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
-                
+
     def get_em_index_price(self, start_date, end_date, index_id_list: Tuple = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -356,7 +361,7 @@ class RawDataApi:
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmIndexPrice.__tablename__}')
                 return None
 
     def delete_em_index_price(self, index_id_list, start_date, end_date):
@@ -372,7 +377,7 @@ class RawDataApi:
                 db_session.commit()
                 return True
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to delete data <err_msg> {e} from {EmIndexPrice.__tablename__}')
                 return None
 
     def get_em_index_val(self, start_date, end_date, index_id_list: Tuple[str] = ()):
@@ -391,7 +396,7 @@ class RawDataApi:
                 )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmIndexVal.__tablename__}')
                 return None
 
     def get_em_fund_scale(self, start_date: str = '', end_date: str = ''):
@@ -561,7 +566,7 @@ class RawDataApi:
                 print(f'Failed to get data <err_msg> {e} from {EmStockEstimateFac.__tablename__}')
                 return None
 
-    def get_em_index_info(self, index_list: Tuple[str] = ()):
+    def get_em_index_info(self, index_list: Union[Tuple[str], List[str]] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
@@ -638,7 +643,7 @@ class RawDataApi:
                     query = query.order_by(EmFundList.datetime.desc()).limit(limit)
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmFundList.__tablename__}')
                 return None
 
     def delete_em_fund_info(self, funds: List[str]):
@@ -650,17 +655,19 @@ class RawDataApi:
             ).delete(synchronize_session=False)
             db_session.commit()
 
-    def get_em_fund_info(self, funds: List[str]):
+    def get_em_fund_info(self, funds: List[str] = []):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                     EmFundInfo
-                ).filter(
-                    EmFundInfo.CODES.in_(funds),
                 )
+                if funds:
+                    query = query.filter(
+                        EmFundInfo.CODES.in_(funds),
+                    )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmFundInfo.__tablename__}')
                 return None
 
     def delete_em_fund_fee(self, funds: List[str]):
@@ -682,7 +689,7 @@ class RawDataApi:
                 )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmFundFee.__tablename__}')
                 return None
 
     def get_em_fund_benchmark(self, end_date: str):
@@ -695,7 +702,7 @@ class RawDataApi:
                 )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EmFundBenchmark.__tablename__}')
                 return None
 
     def get_wind_holder_structure(self, start_date: str, wind_fund_list:list):
@@ -710,22 +717,23 @@ class RawDataApi:
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindFundHolderStructure.__tablename__}')
                 return None
 
-    def get_wind_fund_stock_portfolio(self, start_date: str, wind_fund_list:list):
+    def get_wind_fund_stock_portfolio(self, start_date: str, end_date: str, wind_fund_list: List[str]):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                     WindFundStockPortfolio
                 ).filter(
                     WindFundStockPortfolio.F_PRT_ENDDATE >= start_date,
+                    WindFundStockPortfolio.F_PRT_ENDDATE <= end_date,
                     WindFundStockPortfolio.S_INFO_WINDCODE.in_(wind_fund_list),
                 )
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindFundStockPortfolio.__tablename__}')
                 return None
 
     def get_wind_fund_nav(self, start_date: str, wind_fund_list:list):
@@ -740,7 +748,7 @@ class RawDataApi:
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindFundNav.__tablename__}')
                 return None
 
     def get_wind_manager_info(self, wind_fund_list:list):
@@ -754,7 +762,7 @@ class RawDataApi:
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindFundManager.__tablename__}')
                 return None
 
     def get_wind_indus_portfolio(self, start_date: str, wind_fund_list:list):
@@ -769,59 +777,147 @@ class RawDataApi:
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {WindIndPortfolio.__tablename__}')
                 return None
-    
+
     def get_em_fund_rate(self, datetime: str):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(
                     EMFundRate
                 ).filter(
-                    EMFundRate.DATES == datetime, 
+                    EMFundRate.DATES == datetime,
                 )
                 df = pd.read_sql(query.statement, query.session.bind)
                 return df
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EMFundRate.__tablename__}')
                 return None
 
-    def get_em_fund_hold_asset(self):
+    def get_em_fund_hold_asset(self, start_date: str = '', end_date: str = ''):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(EMFundHoldAsset)
-                em_hold_asset = pd.read_sql(query.statement, query.session.bind)
-                return em_hold_asset
+                if start_date:
+                    query = query.filter(
+                        EMFundHoldAsset.DATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EMFundHoldAsset.DATES <= end_date,
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EMFundHoldAsset.__tablename__}')
                 return None
 
-    def get_em_fund_hold_industry(self):
+    def get_em_fund_hold_industry(self, end_date: str = ''):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(EMFundHoldIndustry)
-                em_hold_stock = pd.read_sql(query.statement, query.session.bind)
-                return em_hold_stock
+                if end_date:
+                    query = query.filter(
+                        EMFundHoldIndustry.DATES <= end_date
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
+                print(f'Failed to get data <err_msg> {e} from {EMFundHoldIndustry.__tablename__}')
                 return None
 
-    def get_em_fund_hold_stock(self):
+    def get_em_fund_hold_stock(self, end_date: str = ''):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(EMFundHoldStock)
-                em_hold_stock = pd.read_sql(query.statement, query.session.bind)
-                return em_hold_stock
+                if end_date:
+                    query = query.filter(
+                        EMFundHoldStock.DATES <= end_date
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EMFundHoldStock.__tablename__}')
+                return None
+
+    def get_em_fund_hold_bond(self, end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EMFundHoldBond)
+                if end_date:
+                    query = query.filter(
+                        EMFundHoldBond.DATES <= end_date
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
-    def get_em_fund_hold_bond(self):
-        with RawDatabaseConnector().managed_session() as raw_session:
+    def get_em_mng_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
             try:
-                query = raw_session.query(EMFundHoldBond)
-                em_hold_bond = pd.read_sql(query.statement, query.session.bind)
-                return em_hold_bond
+                query = db_session.query(
+                    EMMngInfo
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
-                return None
\ No newline at end of file
+
+    def get_em_fund_mng_change(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EMFundMngChange
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+
+    def get_fund_com_mng_change(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EMFundCompMngChange
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+
+    def get_em_fund_com_core_mng(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EMFundCompCoreMng
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+
+    def em_stock_dividend(self, end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmStockDividend)
+                if end_date:
+                    query = query.filter(
+                        EmStockDividend.DATES <= end_date
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockDividend.__tablename__}')
+
+    def em_bond_rating(self, end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmBondRating)
+                if end_date:
+                    query = query.filter(
+                        EmBondRating.DATES <= end_date
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmBondRating.__tablename__}')
