@@ -4,9 +4,9 @@ import traceback
 import json
 import numpy as np
 import pandas as pd
-from typing import List, Dict
+from typing import List, Dict, Optional, Tuple, Union
 
-from ....constant import SectorType, INDEX_PRICE_EXTRA_TRADE_DAYS
+from ....constant import SectorType, IndexPriceSource, INDEX_PRICE_EXTRA_TRADE_DAYS
 from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.basic_models import *
@@ -15,8 +15,11 @@ from .basic_data_helper import BasicDataHelper
 
 class BasicDataPart1:
 
+    _REPORT_DATE_LIST = ['0331', '0630', '0930', '1231']
+
     def __init__(self, data_helper: BasicDataHelper):
         self._data_helper = data_helper
+        self._raw_data_api = RawDataApi()
         self._basic_data_api = BasicDataApi()
         self._index_info_df = self._basic_data_api.get_index_info()
         self._fund_info_df = self._basic_data_api.get_fund_info()
@@ -65,8 +68,8 @@ class BasicDataPart1:
         # Update manually
         # Not verified
         track_index_df = pd.read_csv('./data/fund_track_index.csv', index_col=0)
-        wind_fund_info = RawDataApi().get_wind_fund_info()
-        fund_fee = RawDataApi().get_fund_fee()
+        wind_fund_info = self._raw_data_api.get_wind_fund_info()
+        fund_fee = self._raw_data_api.get_fund_fee()
         wind_fund_info['order_book_id'] = [_.split('!')[0].split('.')[0] for _ in wind_fund_info['wind_id'] ]
         res = []
         for i in wind_fund_info['wind_id']:
@@ -96,6 +99,20 @@ class BasicDataPart1:
         df = pd.read_csv('./data/index_info.csv')
         self._data_helper._upload_basic(df, IndexInfo.__table__.name)
 
+    def index_info_from_raw(self, index_list: Union[Tuple[str], List[str]], index_ids: List[str]):
+        assert len(index_list) == len(index_ids), 'index list length should match the ids'
+
+        df = self._raw_data_api.get_em_index_info(index_list).drop(columns=['name', 'publish_date', '_update_time'])
+        assert df.shape[0] == len(index_ids), 'index info length should match the ids'
+        df = df.rename(columns={'short_name': 'desc_name'})
+        # 先全体置为false，后续可以手工修改
+        df['is_select'] = False
+        df['tag_method'] = 'PE百分位'
+        df['price_source'] = df['price_source'].transform(lambda x: x.name)
+        # index_id是人工起的，所以只能作为参数传进来
+        df['index_id'] = index_ids
+        self._data_helper._upload_basic(df, IndexInfo.__table__.name)
+
     @staticmethod
     def _lambda_func_index_component_2(em_id_to_index_id: Dict[str, str], x: str):
         info = json.loads(x)
@@ -121,23 +138,21 @@ class BasicDataPart1:
         return json.dumps(r)
 
     def index_component(self):
-        api = BasicDataApi()
-        index_info_df = api.get_index_info()
-        index_info_df = index_info_df[index_info_df.em_id.notna()]
+        index_info_df = self._index_info_df[self._index_info_df.em_id.notna()]
         index_info_df = index_info_df.set_index('em_id')
         em_id_to_index_id: Dict[str, str] = index_info_df['index_id'].to_dict()
 
-        df = RawDataApi().get_cs_index_component().drop(columns=['_update_time'])
+        df = self._raw_data_api.get_cs_index_component().drop(columns=['_update_time'])
         df['index_id'] = df['index_id'].map(em_id_to_index_id)
         df['sector'] = df['sector'].transform(lambda x: BasicDataPart1._lambda_func_index_component_2(em_id_to_index_id, x))
         df['id_cat'] = df['id_cat'].transform(lambda x: x.name)
-        fund_info_df = api.get_fund_info().set_index('wind_id')
+        fund_info_df = self._fund_info_df.set_index('wind_id')
         wind_id_to_id: Dict[str, str] = fund_info_df['fund_id'].to_dict()
         df['related_funds'] = df['related_funds'].transform(lambda x: BasicDataPart1._lambda_func_index_component_3(wind_id_to_id, x))
         self._data_helper._upload_basic(df, IndexComponent.__table__.name)
 
     def fund_nav_from_rq(self, start_date, end_date):
-        df = RawDataApi().get_rq_fund_nav(start_date, end_date)
+        df = self._raw_data_api.get_rq_fund_nav(start_date, end_date)
         df['fund_id'] = df.apply(
             lambda x: self._data_helper._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
         df['subscribe_status'] = df['subscribe_status'].map(self._status_mapping)
@@ -150,7 +165,7 @@ class BasicDataPart1:
             start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
             del_date = start_date + datetime.timedelta(days=1)
             start_date = start_date.strftime('%Y%m%d')
-            df = RawDataApi().get_em_fund_nav(start_date, end_date)
+            df = self._raw_data_api.get_em_fund_nav(start_date, end_date)
             df['fund_id'] = df.apply(
                 lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
             df = df.drop(['CODES'], axis=1)
@@ -181,7 +196,7 @@ class BasicDataPart1:
 
     def fund_benchmark(self):
         df = pd.read_csv('./data/benchmark_res2.csv')
-        df = df[['fund_id', 'fund_em_id', 'benchmark', 'index_text',   'assets', 'industry']].rename(columns={'fund_em_id': 'em_id'})
+        df = df[['fund_id', 'fund_em_id', 'benchmark', 'index_text', 'assets', 'industry']].rename(columns={'fund_em_id': 'em_id'})
         self._data_helper._upload_basic(df, FundBenchmark.__table__.name)
 
     def index_price(self, start_date, end_date):
@@ -196,14 +211,12 @@ class BasicDataPart1:
             del_date = trading_day_df.datetime.array[-(INDEX_PRICE_EXTRA_TRADE_DAYS-1)]
             print(f'[index_price] (start_date){start_date} (del_date){del_date} (end_date){end_date}')
             # 准备多个数据源
-            api = RawDataApi()
-            cm_index_price = api.get_raw_cm_index_price_df(start_date, end_date).drop(columns=['_update_time']).set_index('datetime').fillna(method='ffill')
-            cxindex_index_price = api.get_cxindex_index_price_df(start_date, end_date).drop(columns=['_update_time'])
-            yahoo_index_price = api.get_yahoo_index_price_df(start_date, end_date).drop(columns=['_update_time'])
+            cm_index_price = self._raw_data_api.get_raw_cm_index_price_df(start_date, end_date).drop(columns=['_update_time']).set_index('datetime').fillna(method='ffill')
+            cxindex_index_price = self._raw_data_api.get_cxindex_index_price_df(start_date, end_date).drop(columns=['_update_time'])
+            yahoo_index_price = self._raw_data_api.get_yahoo_index_price_df(start_date, end_date).drop(columns=['_update_time'])
             index_info_df = self._index_info_df[self._index_info_df.price_source == IndexPriceSource.default]
             em_id_list: Dict[str, str] = index_info_df[['em_id', 'index_id']].dropna().set_index('em_id').to_dict()['index_id']
-            em_index_price = api.get_em_index_price(start_date, end_date, em_id_list.keys()).drop(columns=['_update_time'])
-
+            em_index_price = self._raw_data_api.get_em_index_price(start_date, end_date, em_id_list.keys()).drop(columns=['_update_time'])
             df_list = []
             # 处理Choice数据
             index_id_list = []
@@ -247,7 +260,7 @@ class BasicDataPart1:
 
     def fund_rating_latest(self):
         try:
-            fund_rating_df = RawDataApi().get_fund_rating()
+            fund_rating_df = self._raw_data_api.get_fund_rating()
             fund_info = self._fund_info_df[['fund_id', 'order_book_id', 'start_date', 'end_date']]
             score_name = ['zs', 'sh3', 'sh5', 'jajx']
             score_dic = {}
@@ -282,8 +295,8 @@ class BasicDataPart1:
 
     def rq_stock_price(self, start_date, end_date):
         try:
-            stock_price_df = RawDataApi().get_rq_stock_price(start_date, end_date)
-            stock_post_price_df = RawDataApi().get_rq_stock_post_price(start_date, end_date)
+            stock_price_df = self._raw_data_api.get_rq_stock_price(start_date, end_date)
+            stock_post_price_df = self._raw_data_api.get_rq_stock_post_price(start_date, end_date)
 
             stock_post_price_df['adj_close'] = stock_post_price_df['close']
             stock_post_price_df = stock_post_price_df.filter(items=['adj_close', 'datetime', 'order_book_id'],
@@ -316,7 +329,7 @@ class BasicDataPart1:
             }
             column_drop_list = ['order_book_id', 'year_to_date_return', 'annualized_risk']
 
-            df = RawDataApi().get_rq_fund_indicator(start_date, end_date)
+            df = self._raw_data_api.get_rq_fund_indicator(start_date, end_date)
             df['fund_id'] = df.apply(
                 lambda x: self._data_helper._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
             df = df.rename(columns=column_rename_dict).drop(columns=column_drop_list)
@@ -331,7 +344,7 @@ class BasicDataPart1:
 
     def stock_info(self):
         try:
-            df = RawDataApi().get_stock_info()
+            df = self._raw_data_api.get_stock_info()
             self._data_helper._upload_basic(df, StockInfo.__table__.name, to_truncate=True)
             return True
         except Exception as e:
@@ -359,8 +372,7 @@ class BasicDataPart1:
 
     def sector_funds(self):
         try:
-            index_info_df = BasicDataApi().get_index_info()
-            index_info_df = index_info_df[index_info_df.em_id.notna()]
+            index_info_df = self._index_info_df[self._index_info_df.em_id.notna()]
             index_info_df = index_info_df.set_index('em_id')
             em_id_list: List[str] = index_info_df.index.to_list()
             em_id_to_index_id: Dict[str, str] = index_info_df['index_id'].to_dict()
@@ -378,10 +390,12 @@ class BasicDataPart1:
     def fund_size(self, start_date, end_date):
         try:
             # Since fund_size in basic db is latest snapshot, we only use end_date as param
-            df = RawDataApi().get_em_fund_scale(end_date=end_date)
+            df = self._raw_data_api.get_em_fund_scale(end_date=end_date)
             if df is None:
                 print('failed to get em fund scale for fund size')
                 return False
+            # 过滤掉非.OF的基金
+            df = df[df.CODES.str.contains('.OF')]
             df = df.pivot_table(index='DATES', columns='CODES', values='FUNDSCALE').fillna(method='ffill')
             df = df.iloc[[-1], :].apply(lambda x: pd.Series({'fund_id': self._data_helper._get_fund_id_from_order_book_id(x.name.split('.')[0], x.index.array[0]), 'latest_size': float(x.array[0])})).T
             df = df.reset_index(drop=True)
@@ -393,19 +407,44 @@ class BasicDataPart1:
             traceback.print_exc()
             return False
 
+    def fund_status_latest(self, end_date: str):
+        try:
+            df: Optional[pd.DataFrame] = self._raw_data_api.get_em_fund_status(end_date=end_date)
+            if df is None:
+                print('failed to get em fund scale for fund status latest')
+                return False
+            # 过滤掉非.OF的基金
+            df = df[df.CODES.str.contains('.OF')]
+            df = df.pivot(index='DATES', columns='CODES', values=['PURCHSTATUS', 'REDEMSTATUS', 'LPLIMIT', 'OTCLPLIMIT', 'TRADESTATUS']).fillna(method='ffill')
+            df = df.iloc[[-1], :].stack().reset_index()
+            df['fund_id'] = df[['DATES', 'CODES']].apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
+            df = df[df.fund_id.notna()]
+            df = df.drop(columns=['DATES', 'CODES']).rename(columns=lambda x: FundStatusLatest.__getattribute__(FundStatusLatest, x).name)
+            self._data_helper._upload_basic(df, FundStatusLatest.__table__.name, to_truncate=True)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
+
     def fund_rate(self, end_date):
         def _count_star_history(x):
             color_change = {
                 '红':'蓝',
                 '蓝':'白',
-                '白':'红',    
+                '白':'红',
             }
             if x is None:
                 return None
             if '星' in x:
-                return str(x.count('★') + x.count('☆') + 1) + color_change[x[-2]] 
+                return str(x.count('★') + x.count('☆') + 1) + color_change[x[-2]]
             else:
-                return str(x.count('★') + x.count('☆')) 
+                return str(x.count('★') + x.count('☆'))
+
+        def _count_A_rating(x: Optional[str]) -> Optional[str]:
+            if x is None:
+                return
+            return str(x.count('A'))
 
         def _count_star_recent(x):
             if x is None:
@@ -413,15 +452,19 @@ class BasicDataPart1:
             if '星' in x:
                 return str(x.count('★') + x.count('☆') + 1) + x[-2]
             else:
-                return str(x.count('★') + x.count('☆')) 
+                return str(x.count('★') + x.count('☆'))
         try:
-            df = RawDataApi().get_em_fund_rate(end_date)
+            df = self._raw_data_api.get_em_fund_rate(end_date)
             df['fund_id'] = df.apply( lambda x : self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
             no_star_columns = ['CODES','DATES','_update_time','fund_id']
+            A_rate_columns = ['TXCOMRATRAT']
             for col in df:
                 if col in no_star_columns:
-                    continue 
-                df[col] = df[col].map(lambda x : _count_star_recent(x))  
+                    continue
+                if col in A_rate_columns:
+                    df[col] = df[col].map(_count_A_rating)
+                else:
+                    df[col] = df[col].map(lambda x: _count_star_recent(x))
             df = df.drop(['CODES','_update_time'], axis=1)
             name_dic = {
                 'DATES':'datetime',
@@ -451,20 +494,33 @@ class BasicDataPart1:
             }
             df = df.rename(columns = name_dic)
             df = df.dropna(subset=['fund_id','datetime'], axis=0)
+            # 寻找离目前最近且没超过当前时间的财报日期，将数据写在这个日期上（先删掉这个日期上的旧数据）
+            end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True)
+            md = end_date_dt.strftime('%m%d')
+            sentry_date = pd.Series(self._REPORT_DATE_LIST)
+            sentry = sentry_date[sentry_date <= md]
+            if sentry.empty:
+                # 这里取前一年的12月31
+                real_date = datetime.date(end_date_dt.year - 1, 12, 31)
+            else:
+                real_date = pd.to_datetime(str(end_date_dt.year)+sentry.array[-1], infer_datetime_format=True).date()
+            df['datetime'] = real_date
+            # TODO: 最好原子提交下边两步
+            BasicDataApi().delete_fund_rate(real_date, df.fund_id.to_list())
             self._data_helper._upload_basic(df, FundRate.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_exc()
             return False
-            
+
     def trading_day_list(self, start_date, end_date, is_automatic_update=True):
         try:
             # 如果是每日自动更新，我们需要去存T+1日的数据
             if is_automatic_update:
                 start_date = end_date
                 end_date = ''
-            df = RawDataApi().get_em_tradedates(start_date, end_date)
+            df = self._raw_data_api.get_em_tradedates(start_date, end_date)
             df = df.rename(columns={'TRADEDATES': 'datetime'})
             if is_automatic_update:
                 df = df.iloc[[1], :]
@@ -478,7 +534,7 @@ class BasicDataPart1:
     def fund_size_hold_rate_history(self):
         start_date = '20100101'
         end_date = '20200612'
-        raw = RawDataApi()
+        raw = self._raw_data_api
         basic = BasicDataApi()
         fund_holding_rate = raw.get_em_fund_holding_rate(start_date=start_date)
         fund_scale = raw.get_em_fund_scale(start_date=start_date, end_date=end_date)
@@ -486,10 +542,15 @@ class BasicDataPart1:
             lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
         fund_scale['fund_id'] = fund_scale.apply(
             lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
-        fund_holding_rate = fund_holding_rate.drop(['CODES', '_update_time'], axis=1).rename(columns={'DATES':'datetime','HOLDPERSONALHOLDINGPCT':'personal_holds','HOLDINSTIHOLDINGPCT':'institution_holds'})
+        fund_holding_rate = fund_holding_rate.drop(['CODES', '_update_time'], axis=1).rename(columns={
+            'DATES':'datetime',
+            'HOLDPERSONALHOLDINGPCT':'personal_holds',
+            'HOLDINSTIHOLDINGPCT':'institution_holds',
+            'HOLDNUM': 'hold_num',
+        })
         fund_scale = fund_scale.drop(['CODES','_update_time'], axis=1).rename(columns={'DATES':'datetime','FUNDSCALE':'size'})
-        fund_scale = fund_scale.dropna().drop_duplicates(subset=['size','fund_id'],keep='first')
-        fund_holding_rate = fund_holding_rate.dropna()
+        fund_scale = fund_scale.dropna(how='all').drop_duplicates(subset=['size','fund_id'],keep='first')
+        fund_holding_rate = fund_holding_rate.dropna(how='all')
         fund_list = list(set(fund_scale.fund_id))
         res = []
         for fund_id in fund_list:
@@ -505,27 +566,29 @@ class BasicDataPart1:
         df = df[con_list]
         self._data_helper._upload_basic(df, Fund_size_and_hold_rate.__table__.name)
 
-    def fund_hold_asset(self):
-        #history, update only on report date
-        em_hold_asset = RawDataApi().get_em_fund_hold_asset()
-        em_hold_asset['CODES'] = em_hold_asset.apply(lambda x : 
+    def fund_hold_asset(self, start_date: str, end_date: str):
+        # history, update only on report date
+        em_hold_asset = self._raw_data_api.get_em_fund_hold_asset(start_date, end_date)
+        em_hold_asset['CODES'] = em_hold_asset.apply(lambda x:
                             self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
         em_hold_asset = em_hold_asset.dropna(subset=['CODES']).drop('_update_time', axis=1)
         name_dict = {
-            'CODES':'fund_id',
-            'DATES':'datetime',
-            'PRTSTOCKTONAV':'stock_nav_ratio',
-            'PRTBONDTONAV':'bond_nav_ratio',
-            'PRTCASHTONAV':'cash_nav_ratio',
-            'PRTOTHERTONAV':'other_nav_ratio',
+            'CODES': 'fund_id',
+            'DATES': 'datetime',
+            'PRTSTOCKTONAV': 'stock_nav_ratio',
+            'PRTBONDTONAV': 'bond_nav_ratio',
+            'PRTCASHTONAV': 'cash_nav_ratio',
+            'PRTOTHERTONAV': 'other_nav_ratio',
+            'MMFFIRSTREPOTONAV': 'first_repo_to_nav',
+            'MMFAVGPTM': 'avg_ptm',
         }
         em_hold_asset = em_hold_asset.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_asset, 'fund_hold_asset')
-        
-    def fund_hold_industry(self):
+        self._data_helper._upload_basic(em_hold_asset, FundHoldAsset.__table__.name)
+
+    def fund_hold_industry(self, end_date: str):
         #history, update only on report date
-        em_hold_industry = RawDataApi().get_em_fund_hold_industry()
-        em_hold_industry['CODES'] = em_hold_industry.apply(lambda x : 
+        em_hold_industry = self._raw_data_api.get_em_fund_hold_industry(end_date)
+        em_hold_industry['CODES'] = em_hold_industry.apply(lambda x :
                             self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
         em_hold_industry = em_hold_industry.dropna(subset=['CODES']).drop('_update_time', axis=1)
         name_dict = {
@@ -533,12 +596,12 @@ class BasicDataPart1:
             'DATES':'datetime',
         }
         em_hold_industry = em_hold_industry.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_industry, 'fund_hold_industry')
+        self._data_helper._upload_basic(em_hold_industry, FundHoldIndustry.__table__.name)
 
-    def fund_hold_stock(self):
+    def fund_hold_stock(self, end_date: str):
         #history, update only on report date
-        em_hold_stock = RawDataApi().get_em_fund_hold_stock()
-        em_hold_stock['CODES'] = em_hold_stock.apply(lambda x : 
+        em_hold_stock = self._raw_data_api.get_em_fund_hold_stock(end_date)
+        em_hold_stock['CODES'] = em_hold_stock.apply(lambda x :
                             self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
         em_hold_stock = em_hold_stock.dropna(subset=['CODES']).drop('_update_time', axis=1)
         name_dict = {
@@ -546,12 +609,12 @@ class BasicDataPart1:
             'DATES':'datetime',
         }
         em_hold_stock = em_hold_stock.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_stock, 'fund_hold_stock')
+        self._data_helper._upload_basic(em_hold_stock, FundHoldStock.__table__.name)
 
-    def fund_hold_bond(self):
+    def fund_hold_bond(self, end_date: str):
         #history, update only on report date
-        em_hold_bond = RawDataApi().get_em_fund_hold_bond()
-        em_hold_bond['CODES'] = em_hold_bond.apply(lambda x : 
+        em_hold_bond = self._raw_data_api.get_em_fund_hold_bond(end_date)
+        em_hold_bond['CODES'] = em_hold_bond.apply(lambda x :
                             self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
         em_hold_bond = em_hold_bond.dropna(subset=['CODES']).drop('_update_time', axis=1)
         name_dict = {
@@ -559,7 +622,32 @@ class BasicDataPart1:
             'DATES':'datetime',
         }
         em_hold_bond = em_hold_bond.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_bond, 'fund_hold_bond')
+        self._data_helper._upload_basic(em_hold_bond, FundHoldBond.__table__.name)
+
+    def mng_info(self):
+        df = self._raw_data_api.get_em_mng_info()
+        df['fund_id'] = df.apply(lambda x : 
+                    self._data_helper._get_fund_id_from_order_book_id(x.code.split('.')[0], datetime.date(2020,8,10)), axis=1)
+        df = df.drop('code',axis=1).dropna(subset=['fund_id'])
+        self._data_helper._upload_basic(df, MngInfo.__table__.name)
+
+    def mng_change(self):
+        df = self._raw_data_api.get_em_fund_mng_change()
+        df['fund_id'] = df.apply(lambda x : 
+                    self._data_helper._get_fund_id_from_order_book_id(x.code.split('.')[0], datetime.date(2020,8,10)), axis=1)
+        df = df.drop('code',axis=1).dropna(subset=['fund_id'])
+        self._data_helper._upload_basic(df, FundMngChange.__table__.name)
+
+    def fund_comp_mng_change(self):
+        df = self._raw_data_api.get_fund_com_mng_change()
+        self._data_helper._upload_basic(df, FundCompMngChange.__table__.name)
+
+    def fund_comp_core_mng(self):
+        df = self._raw_data_api.get_em_fund_com_core_mng()
+        df['fund_id'] = df.apply(lambda x : 
+                    self._data_helper._get_fund_id_from_order_book_id(x.code.split('.')[0], datetime.date(2020,8,10)), axis=1)
+        df = df.drop('code',axis=1).dropna(subset=['fund_id'])
+        self._data_helper._upload_basic(df, FundCompCoreMng.__table__.name)
 
     def process_all(self, start_date, end_date):
         failed_tasks = []
@@ -593,6 +681,10 @@ class BasicDataPart1:
 
         if not self.fund_rate(end_date):
             failed_tasks.append('fund_rate')
+
+        if not self.fund_status_latest(end_date):
+            failed_tasks.append('fund_status_latest')
+
         return failed_tasks
 
 if __name__ == '__main__':
@@ -610,3 +702,4 @@ if __name__ == '__main__':
     # bdp.sector_info()
     # bdp.sector_funds()
     # bdp.index_component()
+    # bdp.fund_rate('20200812')
