@@ -49,7 +49,8 @@ class EmRawDataDownloader:
             print('EM login failed')
             exit()
 
-        self._scale_change_dates = ['0331', '0630', '0930', '1231']
+        self._quarter_update_dates = ['0331', '0630', '0930', '1231']
+        self._semi_update_dates = ['0630', '1231']
 
     def __del__(self):
         c.stop()
@@ -79,7 +80,7 @@ class EmRawDataDownloader:
                     print(f'{index}: {filename}')
                     index += 1
                     fund_nav = pd.concat(pd.read_excel(os.path.join(nav_file_dir, filename), sheet_name=None),
-                                        ignore_index=True)
+                                         ignore_index=True)
                     fund_nav = fund_nav.drop(['简称'], axis=1)
                     fund_nav = fund_nav.rename(columns={
                         '代码': 'CODES',
@@ -135,7 +136,7 @@ class EmRawDataDownloader:
 
                 for d in dates:
                     month_day = d.strftime('%m%d')
-                    if month_day not in self._scale_change_dates:
+                    if month_day not in self._quarter_update_dates:
                         continue
 
                     print(d)
@@ -159,45 +160,66 @@ class EmRawDataDownloader:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
-    def em_fund_holding_rate_history(self):
-        # 机构持有比例半年更新，到期日未必有数据，会有延迟， 含已终止基金
-        fund_id_result = c.sector('507010', '2020-06-10')
-        fund_list = fund_id_result.Codes
-        fund_id_str = ','.join(fund_list)
-        y_list = [str(y) for y in range(2010, 2020,1)]
+    def em_fund_holding_rate_history(self, end_date: str):
+        fund_id_result = c.sector('507010', end_date)
+        if fund_id_result.ErrorCode != 0:
+            print(f'Failed to get fund id list: {fund_id_result.ErrorMsg}')
+            return
+        y_list = [str(y) for y in range(2020, 2021, 1)]
         d_list = []
         for y in y_list:
-            for d in ['-06-30', '-12-31']:
-                d_list.append(y+d)
+            for d in self._semi_update_dates:
+                d_list.append(y + d)
         for d in d_list:
-            data=c.css(fund_id_str,"HOLDPERSONALHOLDINGPCT,HOLDINSTIHOLDINGPCT",f"ReportDate={d} ,Ispandas=1")
+            data = c.css(fund_id_result.Codes, "HOLDPERSONALHOLDINGPCT,HOLDINSTIHOLDINGPCT,HOLDNUM", f"ReportDate={d},Ispandas=1")
             data['DATES'] = d
-            data = data.reset_index().dropna()
+            data = data.dropna(how='all').reset_index()
             self._data_helper._upload_raw(data, EmFundHoldingRate.__table__.name)
-            print(f'finish {d} ')
+            print(f'finish {d}')
 
-    def em_fund_nav(self, start_date, end_date, fund_id_list=()):
+    def em_fund_nav(self, start_date, end_date, fund_id_list: Tuple[str] = ()):
         try:
             # TODO: 应该使用每一天当天的成分列表, 可能所有的sector都需要这样
             # Get all fund ids, 功能函数-板块成分
             # http://quantapi.eastmoney.com/Cmd/Sector?from=web
             start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
             start_date = start_date.strftime('%Y%m%d')
+            trade_date_df = RawDataApi().get_em_tradedates(start_date, end_date)
+            if trade_date_df is None:
+                print('failed to get em trade dates for em_fund_nav')
+                return
             if not fund_id_list:
-                # 这里删掉之前几天的数据，然后再重刷一遍
-                self.delete_em_fund_nav(start_date, end_date)
                 fund_id_result = c.sector('507013', end_date)
                 if fund_id_result.ErrorCode != 0:
                     print(f'Failed to get fund id list: {fund_id_result.ErrorMsg}')
                     return
                 fund_id_list = fund_id_result.Codes
             fund_id_str = ','.join(fund_id_list)
-            df = c.csd(fund_id_str, 'ORIGINALUNIT,ORIGINALNAVACCUM,ADJUSTEDNAV,10KUNITYIELD,YIELDOF7DAYS',
-                       start_date, end_date, 'AdjustFlag=2,Ispandas=1')
-            df = df[df[['ORIGINALUNIT', 'ORIGINALNAVACCUM', 'ADJUSTEDNAV', '10KUNITYIELD', 'YIELDOF7DAYS']].notna().any(axis=1)]
-            # print(df)
-            df = df.rename(columns={'10KUNITYIELD': 'UNITYIELD10K'}).reset_index()
-            self._data_helper._upload_raw(df, EmFundNav.__table__.name)
+            df_list: List[pd.DataFrame] = []
+            for date in trade_date_df.TRADEDATES:
+                # 基金 万份基金单位收益 7日年化收益率
+                data1 = c.css(fund_id_str, "10KUNITYIELD,YIELDOF7DAYS", f"EndDate={date},Ispandas=1")
+                if not isinstance(data1, pd.DataFrame):
+                    if data1.ErrorCode != 0:
+                        print(f'Failed to get fund nav data1: {data1.ErrorMsg}')
+                    return
+                # 基金 单位净值(原始披露) 累计单位净值(原始披露) 复权单位净值
+                data2 = c.css(fund_id_str, "ORIGINALUNIT,ORIGINALNAVACCUM,NAVADJ", f"TradeDate={date},Ispandas=1")
+                if not isinstance(data2, pd.DataFrame):
+                    if data2.ErrorCode != 0:
+                        print(f'Failed to get fund nava data2: {data2.ErrorMsg}')
+                    return
+                df = data2.drop(columns='DATES').join(data1.drop(columns='DATES'), how='outer')
+                df = df[df[['ORIGINALUNIT', 'ORIGINALNAVACCUM', 'NAVADJ', '10KUNITYIELD', 'YIELDOF7DAYS']].notna().any(axis=1)]
+                df = df.rename(columns={'10KUNITYIELD': 'UNITYIELD10K', 'NAVADJ': 'ADJUSTEDNAV'}).reset_index()
+                df['DATES'] = pd.to_datetime(date).date().strftime('%Y-%m-%d')
+                df_list.append(df)
+
+            df_total = pd.concat(df_list)
+            # 这里删掉之前几天的数据，然后再重刷一遍
+            # TODO: 最好可以原子提交下边两步操作
+            self.delete_em_fund_nav(start_date, end_date)
+            self._data_helper._upload_raw(df_total, EmFundNav.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -369,35 +391,86 @@ class EmRawDataDownloader:
             traceback.print_exc()
             return False
 
-    def em_index(self, start_date, end_date):
-        # ----------------------
-        # H11073.CSI,信用债
-        # SPX.GI,标普500
-        # GDAXI.GI,德国DAX
-        # N225.GI,日经225
-        # 000012.SH,利率债
-        # 000300.SH,沪深300
-        # 000905.SH,中证500
-        # 399006.SZ,创业板
-        # 801181.SWI,房地产开发申万
-        # ----------------------
-        # USDCNY.IB,美元兑人民币市询价,TODO: '''ErrorCode=10001012, ErrorMsg=insufficient user access, Data={}'''
-        # JPYCNY.IB,日元兑人民币市询价,TODO: '''ErrorCode=10001012, ErrorMsg=insufficient user access, Data={}'''
-        # EURCNY.IB,欧元兑人民币市询价,TODO: '''ErrorCode=10001012, ErrorMsg=insufficient user access, Data={}'''
-        # ----------------------
-        # AU0.SHF,沪金主力连续,TODO: always get None
-        # BC.ICE,布伦特原油当月连续,TODO: always get None
-        # ----------------------
-        codes = (
-                 'H11073.CSI,SPX.GI,GDAXI.GI,N225.GI,000012.SH,000300.SH,000905.SH,399006.SZ,801181.SWI,'
-                 'USDCNY.IB,JPYCNY.IB,EURCNY.IB,'
-                #  'AU0.SHF,BC.ICE'
-        )
-        df = c.csd(codes, 'CLOSE', start_date, end_date, 'Ispandas=1')
-        if df is not None:
-            df.reset_index(inplace=True)
-            print(df)
-            # self._data_helper._upload_raw(df, EmIndexPrice.__table__.name)
+    def em_realtime_index(self, em_ids: Set[str], data_start_time, to_save):
+        try:
+            # 不支持跨品种的index同时请求，所以需要拆出来分开去做
+            df_list = []
+
+            # 利率部分
+            ir_index = []
+            for one in ('LIGBP1M.IR', 'HIBOR3M.IR'):
+                try:
+                    em_ids.remove(one)
+                except KeyError:
+                    continue
+                ir_index.append(one)
+
+            # 期货部分
+            future_index = []
+            for one in ('AU9999.SGE', 'AU0.SHF'):
+                try:
+                    em_ids.remove(one)
+                except KeyError:
+                    continue
+                future_index.append(one)
+
+            # 常规指数部分
+            data = c.csqsnapshot(','.join(em_ids), 'Date,Time,Now', 'Ispandas=1')
+            if not isinstance(data, pd.DataFrame):
+                if data.ErrorCode != 0:
+                    print(f'[em_index_price] failed to get index price: {data.ErrorMsg} (index_id){em_ids}')
+                return None
+            df_list.append(data)
+
+            # # 每两次 csqsnapshot 请求之间必须大于 3000 ms
+            # # 暂不实时更新利率指数（需要更新时取消注释）
+            # time.sleep(3.1)
+            # if ir_index:
+            #     data = c.csqsnapshot(','.join(ir_index), 'Date,Time,Now', 'Ispandas=1')
+            #     if not isinstance(data, pd.DataFrame):
+            #         if data.ErrorCode != 0:
+            #             print(f'[em_index_price] failed to get index price: {data.ErrorMsg} (index_id){ir_index}')
+            #         return False
+            #     df_list.append(data)
+
+            # # 暂不实时更新期货指数（需要更新时取消注释）
+            # time.sleep(3.1)
+            # if future_index:
+            #     data = c.csqsnapshot(','.join(future_index), 'Date,Time,Now', 'Ispandas=1')
+            #     if not isinstance(data, pd.DataFrame):
+            #         if data.ErrorCode != 0:
+            #             print(f'[em_index_price] failed to get index price: {data.ErrorMsg} (index_id){future_index}')
+            #         return False
+            #     df_list.append(data)
+
+            df = pd.concat(df_list)
+            df = df[df['DATE'] != 0]
+            df = df[df['TIME'] != 0]
+            df['datetime'] = df.apply(
+                lambda x: datetime.datetime.strptime(f'{x["DATE"]}{x["TIME"]}', '%Y%m%d%H%M%S'), 
+                axis=1
+            )
+            if data_start_time:
+                df = df[df['datetime'] > data_start_time]
+
+            df = df.drop(columns=['DATES', 'DATE', 'TIME'])
+            df = df.reset_index().rename(columns={
+                'CODES': 'em_id',
+                'NOW': 'price'
+            })
+            if to_save:
+                try:
+                    self._data_helper._upload_raw(df, EmRealtimeIndexPrice.__table__.name)
+                except Exception as e:
+                    print(e)
+                    traceback.print_exc()
+            
+            return df
+
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return None
 
     # 从em客户端导出数据后读取并处理
     def em_stock_price_history(self, stock_price_file_dir: str):
@@ -654,7 +727,7 @@ class EmRawDataDownloader:
                             'EBIT,EBITDA,EXTRAORDINARY,LOWERDIANDNI,GROSSMARGIN,OPERATEINCOME,INVESTINCOME,EBITDRIVE,TOTALCAPITAL,WORKINGCAPITAL,\
                              NETWORKINGCAPITAL,TANGIBLEASSET,RETAINED,INTERESTLIBILITY,NETLIBILITY,EXINTERESTCL,EXINTERESTNCL,FCFF,FCFE,DA,FCFFDRIVE,\
                              PERFORMANCEEXPRESSPARENTNI,MBSALESCONS,GPMARGIN,NPMARGIN,EXPENSETOOR,INVTURNRATIO,ARTURNRATIO,ROEAVG,ROEWA,EPSBASIC,EPSDILUTED,\
-                             BPS,BALANCESTATEMENT_25,BALANCESTATEMENT_46,BALANCESTATEMENT_93,BALANCESTATEMENT_103,BALANCESTATEMENT_141,BALANCESTATEMENT_195,\
+                             BPS,MBREVENUE,MBCOST,BALANCESTATEMENT_25,BALANCESTATEMENT_46,BALANCESTATEMENT_93,BALANCESTATEMENT_103,BALANCESTATEMENT_141,BALANCESTATEMENT_195,\
                              BALANCESTATEMENT_140,INCOMESTATEMENT_9,INCOMESTATEMENT_48,INCOMESTATEMENT_60,INCOMESTATEMENT_61,INCOMESTATEMENT_83,INCOMESTATEMENT_85,INCOMESTATEMENT_127,\
                              INCOMESTATEMENT_14,CASHFLOWSTATEMENT_39,CASHFLOWSTATEMENT_59,CASHFLOWSTATEMENT_77,CASHFLOWSTATEMENT_82,CASHFLOWSTATEMENT_86',
                             f'ReportDate={end_date},type=1,Ispandas=1')
@@ -859,6 +932,7 @@ class EmRawDataDownloader:
                     print(f'failed to get index val info part 2: {another_data.ErrorMsg} (date){date}')
                 break
             df_list.append(data.join(another_data.drop(columns='DATES'), how='outer'))
+            print(f'{date} done')
 
         if df_list:
             if len(df_list) == 1:
@@ -918,6 +992,7 @@ class EmRawDataDownloader:
                     df = df_list[0]
                 else:
                     df = pd.concat(df_list)
+                df = df[df.drop(columns='DATES').notna().any(axis=1)]
                 df = df.reset_index().rename(columns=lambda x: EmIndexVal.__getattribute__(EmIndexVal, x).name)
                 self._data_helper._upload_raw(df, EmIndexVal.__table__.name)
             return True
@@ -1126,7 +1201,11 @@ class EmRawDataDownloader:
         if not fund_id_list:
             return True
         try:
-            data = c.css(','.join(fund_id_list), 'NAME,FNAME,FUNDTYPE,FOUNDDATE,MATURITYDATE,FIRSTINVESTTYPE,SECONDINVESTTYPE,TRADECUR,STRUCFUNDORNOT,GFCODEM,PREDFUNDMANAGER,MGRCOMP', 'Ispandas=1')
+            # 过滤掉非.OF结尾的基金
+            fund_id_list = [one for one in fund_id_list if one.split('.')[1] == 'OF']
+            if not fund_id_list:
+                return True
+            data = c.css(','.join(fund_id_list), 'NAME,FNAME,FUNDTYPE,FOUNDDATE,MATURITYDATE,FIRSTINVESTTYPE,SECONDINVESTTYPE,TRADECUR,STRUCFUNDORNOT,GFCODEM,PREDFUNDMANAGER,MGRCOMP,CUSTODIANBANK,FOREIGNCUSTODIAN', 'Ispandas=1')
             if not isinstance(data, pd.DataFrame):
                 if data.ErrorCode != 0:
                     print(f'failed to get fund info: {data.ErrorMsg}')
@@ -1198,6 +1277,7 @@ class EmRawDataDownloader:
         df = RawDataApi().get_em_tradedates(start_date, end_date)
         for row in df.itertuples(index=False):
             self.em_fund_benchmark(row.TRADEDATES, fund_list_result.Codes)
+            print(f'[em_fund_benchmark_history] {row.TRADEDATES} done')
 
     def em_fund_benchmark(self, end_date: str, fund_list):
         try:
@@ -1241,16 +1321,26 @@ class EmRawDataDownloader:
             traceback.print_exc()
             return False
 
-    def em_fund_asset(self):
-        all_fund = c.sector('507013', '20200719').Codes
-        dt_list = ['20191231','20200331','20200630']
-        res = []
-        for dt in dt_list:
-            data=c.css(all_fund,"PRTSTOCKTONAV,PRTBONDTONAV,PRTCASHTONAV,PRTOTHERTONAV",f"ReportDate={dt},Ispandas=1")
-            data['DATES'] = dt
-            res.append(data)
+    def em_fund_asset(self, end_date: str):
+        fund_list = RawDataApi().get_em_fund_list(end_date, limit=1)
+        if fund_list is None:
+            print(f'failed to get fund list for fund status')
+            return False
+        fund_list = fund_list.all_live_fund_list.array[0]
+
+        start_year = 2010
+        end_year = 2014
+        res: List[pd.DataFrame] = []
+        for year in range(start_year, end_year+1):
+            for date in self._quarter_update_dates:
+                dt = str(year) + date
+                print(f'doing {dt}')
+                data = c.css(fund_list, "PRTSTOCKTONAV,PRTBONDTONAV,PRTCASHTONAV,PRTOTHERTONAV,MMFFIRSTREPOTONAV,MMFAVGPTM", f"ReportDate={dt},Ispandas=1")
+                data['DATES'] = dt
+                res.append(data)
+                print(f'{dt} done')
         df = pd.concat(res).reset_index()
-        self._data_helper._upload_raw(df, 'em_fund_asset')
+        self._data_helper._upload_raw(df, EMFundHoldAsset.__table__.name)
 
     def em_fund_industry(self):
         all_fund = c.sector('507013', '20200719').Codes
@@ -1258,6 +1348,7 @@ class EmRawDataDownloader:
         res = []
         rank_list = [1,2,3]
         for dt in dt_list:
+            print(f'doing {dt}')
             _res = [] 
             for rank in rank_list:
                 data=c.css(all_fund,"PRTKEYINDNAME,PRTKEYINDTOASSET",f"ReportDate={dt},Rank={rank},IndustryCode=A,NewOldType=1,Ispandas=1")
@@ -1267,8 +1358,20 @@ class EmRawDataDownloader:
             df = pd.concat(_res,axis=1)
             df['DATES'] = dt
             res.append(df)
+            print(f'{dt} done')
         df = pd.concat(res).reset_index()
-        self._data_helper._upload_raw(df, 'em_fund_industry')
+        self._data_helper._upload_raw(df, EMFundHoldIndustry.__table__.name)
+
+    def em_fund_stock_history(self, fund_stock_path: str, report_date: str):
+        df: pd.DataFrame = pd.read_excel(fund_stock_path, na_values='——')
+        df = df[df['证券名称'].notna()]
+        df = df.drop(columns='证券名称')
+
+        columns_name = ['CODES'] + ['rank'+str(i+1)+'_stock'+suffix for suffix in ('', '_code', 'val', 'weight') for i in range(10)]
+        assert len(columns_name) == df.shape[1], 'len of columns name should be equal with columns number of df'
+        df = df.set_axis(columns_name, axis=1)
+        df['DATES'] = report_date
+        em._data_helper._upload_raw(df, EMFundHoldStock.__table__.name)
 
     def em_fund_stock(self):
         all_fund = c.sector('507013', '20200719').Codes
@@ -1293,6 +1396,113 @@ class EmRawDataDownloader:
         df = pd.concat(res).reset_index()
         self._data_helper._upload_raw(df, 'em_fund_stock')
 
+    def em_fund_bond_history(self, fund_bond_path: str, report_date: str):
+        df: pd.DataFrame = pd.read_excel(fund_bond_path, na_values='——')
+        df = df[fund_bond['证券名称'].notna()]
+        df = df.drop(columns='证券名称')
+
+        columns_name = ['CODES'] + ['rank'+str(i+1)+'_bond'+suffix for suffix in ('', '_code', 'val', 'weight') for i in range(10)]
+        assert len(columns_name) == df.shape[1], 'len of columns name should be equal with columns number of df'
+        df = df.set_axis(columns_name, axis=1)
+        df['DATES'] = report_date
+        em._data_helper._upload_raw(df, EMFundHoldBond.__table__.name)
+
+    def em_mng_info_history(self,path):
+        col_dic = {'基金经理':'mng_name', 
+            '代码': 'code', 
+            '名称':'desc_name', 
+            '在管基金规模(亿元)':'fund_size', 
+            '任职日期':'start_date',
+            '任职天数':'work_days', 
+            '任职以来回报(%)':'his_ret',
+            '任职以来年化回报(%)':'his_annual_ret', 
+            '业绩比较基准':'benchmark', 
+            '基金类型':'fund_type', 
+            '管理公司':'company_name', 
+            '共同任职经理':'coworkers', 
+            '基金经理年限':'mng_work_year', 
+            '任职基金数':'worked_funds_num',
+            '任职基金公司数':'worked_funds_num', 
+            '任职基金几何总回报(%)':'fund_total_geo_ret', 
+            '几何平均年化收益率(%)':'geo_annual_ret', 
+            '算术平均年化收益率(%)':'cal_annual_ret', 
+            '性别':'gender', 
+            '出生年份':'birth_year',
+            '年龄':'age',
+            '学历':'education', 
+            '国籍':'nationality', 
+            '简历':'resume', 
+            '最早任职基金经理日期':'start_mng_date'}
+        df = pd.read_excel(path, header=1).dropna(axis=0)
+        df = df.rename(columns = col_dic)
+        df = df.replace('——',np.NaN)
+        em._data_helper._upload_raw(df,EMMngInfo.__table__.name)
+
+    def em_fund_mng_change_history(self, path):
+        col_dic = {
+            '代码':'code', 
+            '名称':'desc_name', 
+            '现任基金经理':'mng_now', 
+            '现任经理最新任职日期':'mng_now_start_date', 
+            '最早任职基金经理':'mng_begin', 
+            '最早任职基金经理日期':'mng_begin_date',
+            '现任基金经理年限':'mng_now_work_year', 
+            '同类型基金现任基金经理年限均值':'same_type_fund_work_year', 
+            '已离任基金经理':'resign_mngs', 
+            '历任基金经理人数':'total_mng_num', 
+            '历任基金经理人均任职年限':'total_mng_avg_work_year',
+            '投资类型':'fund_type',
+            '管理公司':'company', 
+            '监管辖区':'region'
+        }
+        df = pd.read_excel(path, header=0).dropna(axis=0)
+        df = df.rename(columns = col_dic)
+        df = df.replace('——',np.NaN)
+        df = df.drop_duplicates('code') # 发现有重复的数据 比如 codes = 000063.OF
+        em._data_helper._upload_raw(df,EMFundMngChange.__table__.name)
+
+    def em_fund_comp_mng_change_history(self, path):
+        col_dic = {
+            '基金公司':'company',
+            '基金经理数':'total_mng_num',
+            '基金经理平均年限':'mng_avg_year',
+            '基金经理最大年限':'mng_max_year',
+            '团队稳定性':'team_stability',
+            '新聘基金经理数':'new_mng_num',
+            '离职基金经理数':'resign_mng_num',
+            '基金经理变动率(%)':'mng_turnover_rate',
+            '1年以内':'exp_less_than_1',
+            '1-2年':'exp_1_to_2',
+            '2-3年':'exp_2_to_3',
+            '3-4年':'exp_3_to_4',
+            '4年以上':'exp_more_than_4',
+        }
+        df = pd.read_excel(path, header=1).dropna(axis=0)
+        df = df.rename(columns = col_dic)
+        df = df.replace('——',np.NaN)
+        em._data_helper._upload_raw(df,EMFundCompMngChange.__table__.name)
+
+    def em_fund_com_core_mng_history(self, path):
+        col_dic = {
+            '基金代码':'code',
+            '基金名称':'desc_name',
+            '本基金离职基金经理人数':'fund_resign_mng_num',
+            '所属基金公司':'company',
+            '基金公司基金经理数':'com_mng_num',
+            '基金公司离职基金经理数':'com_resign_mng_num',
+            '基金公司基金经理离职率(%)':'com_resign_mng_rate',
+            '基金公司核心人员人数':'com_core_mng_num',
+            '基金公司核心人员离职人数':'com_core_mng_resign_num',
+            '基金公司核心人员离职率(%)':'com_core_mng_resign_rate',
+            '投资类型':'fund_type',
+            '监管区域':'region'
+        }
+        df = pd.read_excel(path, header=1).dropna(axis=0)
+        df = df.rename(columns = col_dic)
+        df = df.replace('——',np.NaN)
+        df = df.drop_duplicates('code') # 发现有重复的数据 比如 codes = 000424.OF
+        em._data_helper._upload_raw(df,EMFundCompCoreMng.__table__.name)
+
     def em_fund_bond(self):
         all_fund = c.sector('507013', '20200719').Codes
         fund_list = all_fund
@@ -1300,6 +1510,7 @@ class EmRawDataDownloader:
         res = []
         rank_list = [i for i in range(1,11,1)]
         for dt in dt_list:
+            print(f'doing {dt}')
             _res = [] 
             for rank in rank_list:
                 data=c.css(fund_list,"PRTKEYBONDNAME,PRTKEYBONDCODE,PRTKEYBONDVALUE,PRTKEYBONDTONAV",f"ReportDate={dt},Rank={rank},Ispandas=1")
@@ -1309,8 +1520,9 @@ class EmRawDataDownloader:
             df = pd.concat(_res,axis=1)
             df['DATES'] = dt
             res.append(df)
+            print(f'{dt} done')
         df = pd.concat(res).reset_index()
-        em._data_helper._upload_raw(df, 'em_fund_bond')
+        em._data_helper._upload_raw(df, EMFundHoldBond.__table__.name)
 
     def download_all(self, start_date, end_date):
         failed_tasks = []
@@ -1395,14 +1607,16 @@ if __name__ == '__main__':
     # em.em_stock_daily_info_history('./')
     # em.em_stock_refinancing('20170101', '20181231')
     # em.add_columns_to_fin_fac('20190930')
-    # em.em_fund_status('20200513', '20200513')
+    # em.em_fund_status('20200513')
     # em.em_index_component('20200528', '20200529')
     # em.em_fund_list('20200702', '20200702')
     # em.em_fund_info_history('20200707')
-    # em.em_fund_benchmark_history('20040101', '20041231')
+    # em.em_fund_benchmark_history('20100101', '20101231')
     # em.em_index_info(['399972.SZ'])
     # em.em_fund_info({'165807.SZ'})
     # em.em_stock_estimate_fac('20200101', '20200709', 2020)
     # em.cs_index_component('20200713')
     # em.em_macroeconomic_price('20200714', '20200714')
     # em.em_fund_fee_history('20200715')
+    # em.em_fund_asset('20200819')
+    # em.em_fund_holding_rate_history('20200819')
\ No newline at end of file
