@@ -5,6 +5,8 @@ import datetime
 
 @dataclasses.dataclass
 class FundDataTables:
+    # raw
+    cs_index_component: pd.DataFrame = None
     # basic
     trading_days: pd.DataFrame = None
     index_info: pd.DataFrame = None
@@ -36,8 +38,8 @@ class FundDataTables:
         return s
 
     def _agg_indicator(self):
+        self.fund_indicator = self.fund_indicator.rename(columns={'var':'var_'})
         self.index_fund_indicator_pack = self.fund_indicator.pivot_table(index=['index_id', 'datetime', 'fund_id'])
-        self.index_fund_indicator_pack['ir'] = self.index_fund_indicator_pack['alpha'] / self.index_fund_indicator_pack['track_err']
         self.index_list = set(self.index_fund_indicator_pack.index.levels[0])
 
     def fix_data(self):
