@@ -77,6 +77,8 @@ def filter_fund_info(fund_info: pd.DataFrame, index_list: List[str], is_normal_t
             & (~fund_info.desc_name.str.contains('短融')) 
             # 去掉双债型债券  双债多含可转债 共41只
             & (~fund_info.desc_name.str.contains('双债'))
+            # 去掉短期纯债型基金
+            & (fund_info.wind_class_2 != '短期纯债型基金')
             # 货币基金总共1000多只，fund indicator 计算压力大，当前人工用2020年5月的货币基金规模取了前50只作为 货币基金池
             | (fund_info.is_selected_mmf == 1)
             # 增加绝对收益型基金 17只
@@ -105,8 +107,11 @@ def fund_info_update(fund_info: pd.DataFrame):
     for r in fund_info.itertuples():
         if (r.benchmark_1) and ('恒生' in r.benchmark_1):
             hsi_fund.append(r.fund_id)
-    fund_info.loc[fund_info.fund_id.isin(hsi_fund), 'index_id'] = 'hsi'
-    
     fund_info.loc[fund_info.fund_id.isin(hsi_fund), 'index_id_new'] = 'hsi'
+
+    #增加混合债型一级基金 到国债下，共271 （混合一级含股票， 股灾时mdd会扩大，还是考虑排除）
+    #hybrid_class_1_funds = fund_info[fund_info['wind_class_2'] == '混合债券型一级基金'].fund_id.to_list()
+    #fund_info.loc[fund_info.fund_id.isin(hybrid_class_1_funds), 'index_id_new'] = 'national_debt'
     fund_info.loc[:,'index_id'] = fund_info['index_id_new']
+
     return fund_info
\ No newline at end of file
