@@ -1,7 +1,8 @@
 
-from typing import Dict, Optional, Union
+from typing import Dict, Optional, Union, Tuple
 from . import DataManager
 import pandas as pd
+import numpy as np
 import datetime
 import time
 import io
@@ -168,7 +169,6 @@ class FundDataManager(DataManager):
                     # FundIndicator.down_risk,
                     # FundIndicator.info_ratio,
                     # FundIndicator.ret_over_period,
-                    # FundIndicator.info_ratio,
                     # FundIndicator.treynor,
                     # FundIndicator.mdd,
                     # FundIndicator.m_square,
@@ -187,7 +187,14 @@ class FundDataManager(DataManager):
             _tm_deriv_fetch_fund_indicator = time.time()
             self.dts.fund_indicator['index_id'] = self.dts.fund_indicator.fund_id.apply(lambda x: self.dts.fund_index_map[x])
             _tm_deriv_apply_fund_indicator = time.time()
-            self.dts.fund_indicator_annual = fetch_table(derived_session, FundIndicatorAnnual)
+            _fund_indicator_query = derived_session.query(
+                FundIndicatorAnnual
+            ).filter(
+                FundIndicatorAnnual.fund_id.in_(self.dts.fund_list),
+                FundIndicatorAnnual.datetime <= self.end_date,
+            )
+            self.dts.fund_indicator_annual = pd.read_sql(_fund_indicator_query.statement, _fund_indicator_query.session.bind)
+            self.dts.fund_indicator_annual = self.dts.fund_indicator_annual.set_index(['fund_id','datetime'])
             _tm_deriv_fetch_fund_indicator_annual = time.time()
             # fetch index val data from derived table, and choose different pct value for different index_id
             _tm_deriv_fetch_index_pct = time.time()
@@ -243,10 +250,23 @@ class FundDataManager(DataManager):
                 for df in [_fund_indicator_weekly, _fund_indicator_monthly]:
                     self.dts.fund_indicator = self.dts.fund_indicator.join(df, on=['fund_id', 'datetime'], how='left')
                 _tm_process_indicator_join = time.time()
+                self.dts.fund_indicator_annual.index.set_levels(self.dts.fund_indicator_annual.index.levels[1].date, level=1, inplace=True)
+                the_dict = self.dts.fund_indicator_annual.to_dict(orient='index')
+                _res = self.dts.fund_indicator[['fund_id', 'datetime']].apply(lambda x : self._make_annual_fac(the_dict, x), axis=1)
+                _tm_process_fund_annual_apply= time.time()
+                _annual_fac_df = pd.DataFrame(_res.tolist())
+                self.dts.fund_indicator = pd.merge(self.dts.fund_indicator,_annual_fac_df,how='left',left_on=['fund_id','datetime'],right_on=['fund_id','datetime'])
+                _tm_process_fund_annual_join = time.time()
             else:
                 _tm_process_indicator_weekly = time.time()
                 _tm_process_indicator_monthly = _tm_process_indicator_weekly
                 _tm_process_indicator_join = _tm_process_indicator_weekly
+                _tm_process_fund_annual_apply = _tm_process_indicator_weekly
+                _tm_process_fund_annual_join = _tm_process_indicator_weekly
+
+            self._barra_cne5_factor_return = fetch_table(derived_session, BarraCNE5FactorReturn)
+            _tm_deriv_fetch_barra_cne5_factor_return = time.time()
+
             if print_time:
                 print(f'\t[time][deriv] fetch fund indicator: {_tm_deriv_fetch_fund_indicator - _tm_deriv_start}')
                 print(f'\t[time][deriv] apply fund indicator: {_tm_deriv_apply_fund_indicator - _tm_deriv_fetch_fund_indicator}')
@@ -256,6 +276,9 @@ class FundDataManager(DataManager):
                 print(f'\t[time][deriv] process indicator weekly: {_tm_process_indicator_weekly - _tm_deriv_apply_index_pct}')
                 print(f'\t[time][deriv] process indicator monthly: {_tm_process_indicator_monthly - _tm_process_indicator_weekly}')
                 print(f'\t[time][deriv] process indicator join: {_tm_process_indicator_join - _tm_process_indicator_monthly}')
+                print(f'\t[time][deriv] process indicator annually apply: {_tm_process_fund_annual_apply - _tm_process_indicator_join}')
+                print(f'\t[time][deriv] merge annual fac: {_tm_process_fund_annual_join - _tm_process_fund_annual_apply}')
+                print(f'\t[time][deriv] fetch barra cne5 factor return:  {_tm_deriv_fetch_barra_cne5_factor_return - _tm_process_fund_annual_join}')
         _tm_derived = time.time()
         self.dts.fix_data()
         _tm_fix = time.time()
@@ -273,6 +296,28 @@ class FundDataManager(DataManager):
             print(f'[time] score: {_tm_finish - _tm_fix}')
             print(f'[time] total: {_tm_finish - _tm_start}')
 
+    def _make_annual_fac(self, the_dict, x):
+        fund_id = x.fund_id
+        dt = x.datetime
+        dic = {'fund_id': fund_id, 'datetime': dt}
+        last_year = datetime.date(dt.year - 1,12,31)
+        last_two_year = datetime.date(dt.year - 2,12,31)
+
+        try:
+            dic['last_year_alpha'] = the_dict[(fund_id, last_year)]['alpha_annual']
+            dic['last_year_beta'] = the_dict[(fund_id, last_year)]['beta_annual']
+            dic['last_two_year_alpha'] = the_dict[(fund_id, last_two_year)]['alpha_annual']
+            dic['last_two_year_beta'] = the_dict[(fund_id, last_two_year)]['beta_annual']
+        except KeyError:
+            pass
+        return dic
+
+    def _get_annual_alpha_beta(self, _fund_indicator_annual_fund_i, year):
+        if year in _fund_indicator_annual_fund_i.index:
+            return _fund_indicator_annual_fund_i.loc[year,'alpha_annual'], _fund_indicator_annual_fund_i.loc[year,'beta_annual']
+        else:
+            return None, None
+
     def set_score_param(self, score_param: FundScoreParam):
         self._score_manager.set_param(score_param)
 
@@ -334,6 +379,16 @@ class FundDataManager(DataManager):
     def get_fund_indicator_annual(self) -> pd.DataFrame:
         return self.dts.fund_indicator_annual
 
+    def fund_analysis(self, fund_id: str, is_plot: bool = False) -> Tuple[np.ndarray, pd.Series]:
+        from ..fund.derived.barra_cne5_factor_processor import BarraCNE5FactorProcessor
+        try:
+            params, spec_ret, fund_ret, factor_ret_contri = BarraCNE5FactorProcessor.fund_analysis(self._barra_cne5_factor_return, self.dts.fund_nav.loc[:, fund_id])
+        except KeyError:
+            return
+        if is_plot:
+            BarraCNE5FactorProcessor.fund_analysis_plot(fund_ret, factor_ret_contri)
+        return params, spec_ret
+
     def white_and_black_list_filter(self, score, score_raw, disproved_list):
         return self._score_manager._white_and_black_list_filter(score, score_raw, disproved_list)
 
