@@ -6,6 +6,8 @@ from .data_tables import FundDataTables
 import dataclasses
 import pandas as pd
 import re
+import datetime
+import time
 import math
 import numpy as np
 from copy import deepcopy
@@ -17,32 +19,36 @@ class ScoreFunc:
     beta: float = 0  # weight of abs(1-beta), or beta's deviation from 1
     track_err: float = 0  # weight of track_err
     fee_rate: float = 0  # weight of fee_rate
-    down_risk: float = 0
-    info_ratio: float = 0
-    ret_over_period: float = 0
-    treynor: float = 0
-    mdd: float = 0
-    m_square: float = 0
-    var_: float = 0
-    sharpe: float = 0
-    annual_ret: float = 0
-    annual_vol: float = 0
-    time_ret: float = 0
-    alpha_w: float = 0
-    beta_w: float = 0
-    track_err_w: float = 0
-    beta_m: float = 0
-    calmar_ratio_m: float = 0
-    information_ratio_m: float = 0
-    jensen_alpha_m: float = 0
-    sharpe_ratio_m: float = 0
-    treynor_ratio_m: float = 0
+    # last_year_alpha: float = 0
+    # last_year_beta: float = 0
+    # last_two_year_alpha: float = 0
+    # last_two_year_beta: float = 0
+    # down_risk: float = 0
+    # info_ratio: float = 0
+    # ret_over_period: float = 0
+    # treynor: float = 0
+    # mdd: float = 0
+    # m_square: float = 0
+    # var_: float = 0
+    # sharpe: float = 0
+    # annual_ret: float = 0
+    # annual_vol: float = 0
+    # time_ret: float = 0
+    # alpha_w: float = 0
+    # beta_w: float = 0
+    # track_err_w: float = 0
+    # beta_m: float = 0
+    # calmar_ratio_m: float = 0
+    # information_ratio_m: float = 0
+    # jensen_alpha_m: float = 0
+    # sharpe_ratio_m: float = 0
+    # treynor_ratio_m: float = 0
 
     def get(self, data):
         return  self.alpha * data.alpha \
                 + self.track_err * data.track_err \
                 + self.fee_rate * data.fee_rate \
-                + self.beta * abs(1-data.beta) 
+                + self.beta * abs(1-data.beta)
                 # + self.down_risk * data.down_risk \
                 # + self.info_ratio * data.info_ratio \
                 # + self.ret_over_period * data.ret_over_period \
@@ -53,8 +59,8 @@ class ScoreFunc:
                 # + self.sharpe * data.sharpe \
                 # + self.annual_ret * data.annual_ret \
                 # + self.annual_vol * data.annual_vol \
-                # + self.time_ret * data.time_ret \ 
-            
+                # + self.time_ret * data.time_ret
+
     def get_all(self, data):
         return  self.alpha * data.alpha \
                 + self.track_err * data.track_err \
@@ -79,85 +85,40 @@ class ScoreFunc:
                 + self.sharpe_ratio_m * data.sharpe_ratio_m \
                 + self.treynor_ratio_m * data.treynor_ratio_m \
                 + self.beta_w * abs(1-data.beta_w) \
-                + self.beta_m * abs(1-data.beta_m)
+                + self.beta_m * abs(1-data.beta_m) \
+                + self.last_year_alpha * data.last_year_alpha \
+                + self.last_year_beta * data.last_year_beta \
+                + self.last_two_year_alpha * data.last_two_year_alpha \
+                + self.last_two_year_beta * data.last_two_year_beta
+
+    def __str__(self):
+        func_str = ''
+        for fac, w in self.__dict__.items():
+            if w == 0:
+                continue
+            flag = ''
+            if w > 0:
+                flag = '+'
+            if fac == 'beta':
+                func_str += f'{flag}{w} * abs(1-{fac})'
+            else:
+                func_str += f'{flag}{w} * {fac}'
+        return func_str
+
 
 class FundScoreManager:
 
     def __init__(self, score_filter=ScoreFilter(), score_penalty_param=ScorePenaltyParams()):
         self.params = None
         self.dts = None
-        # 归一化之后的评分
-        # self.funcs = {
-        #     'hs300': ScoreFunc(alpha=0.3, beta=-0.6, fee_rate=-0.1),
-        #     'csi500': ScoreFunc(alpha=0.8, beta=-0.2),
-        #     'gem': ScoreFunc(alpha=0.05, beta=-0.50, track_err=-0.45),
-        #     'national_debt': ScoreFunc(alpha=0.32, beta=-0.33, track_err=-0.35),
-        #     'mmf': ScoreFunc(alpha=0.6, fee_rate=-0.3, track_err=-0.1),
-        #     'sp500rmb': ScoreFunc(fee_rate=-0.2, track_err=-0.8),
-        #     'gold': ScoreFunc(alpha=0.15, fee_rate=0, track_err=-0.85),
-        # }
-        '''
-        # weekly indicator mean of std on each day each index
-        {'alpha_std': {'csi500': 0.081316,
-                        'gem': 0.060555,
-                        'gold': 0.014478,
-                        'hs300': 0.050694,
-                        'mmf': 0.0063286,
-                        'national_debt': 0.0751936,
-                        'sp500rmb': 0.0163182},
-        'fee_rate_std': {'csi500': 0.0031425,
-                            'gem': 0.0043791,
-                            'gold': 0.0031,
-                            'hs300': 0.002674,
-                            'mmf': 0.00083814,
-                            'national_debt': 0.002321,
-                            'sp500rmb': 0.0022616},
-        'track_err_std': {'csi500': 0.0568387,
-                        'gem': 0.0554576,
-                        'gold': 0.024641,
-                        'hs300': 0.04155,
-                        'mmf': 0.000555,
-                        'national_debt': 0.052457,
-                        'sp500rmb': 0.0140498}}
-        
-        # 日因子均值
-        {'gold': {'alpha_mean': -0.00145, 'fee_mean': 0.00793, 'track_mean': 0.06711},
-        'sp500rmb': {'alpha_mean': -0.0038, 'fee_mean': 0.01029,'track_mean': 0.02805},
-        'mmf': {'alpha_mean': 0.00998, 'fee_mean': 0.00325, 'track_mean': 0.0021},
-        'csi500': {'alpha_mean': 0.02104, 'fee_mean': 0.00806, 'track_mean': 0.07783},
-        'hs300': {'alpha_mean': 0.02763, 'fee_mean': 0.00889, 'track_mean': 0.04522},
-        'national_debt': {'alpha_mean': 0.03921, 'fee_mean': 0.00573, 'track_mean': 0.02394},
-        'gem': {'alpha_mean': -0.02174, 'fee_mean': 0.00868, 'track_mean': 0.05726}}
-
-        # 日因子 std
-        'csi500': {'alpha_std': 0.08291, 'fee_rate': 0.0031, 'track_err': 0.06534},
-        'hs300': {'alpha_std': 0.0444, 'fee_rate': 0.00266, 'track_err': 0.03939},
-        'sp500rmb': {'alpha_std': 0.01625, 'fee_rate': 0.00228, 'track_err': 0.01088},
-        'mmf': {'alpha_std': 0.00917, 'fee_rate': 0.00084, 'track_err': 0.00123},
-        'national_debt': {'alpha_std': 0.08303,'fee_rate': 0.00229,'track_err': 0.05373},
-        'gem': {'alpha_std': 0.04332, 'fee_rate': 0.00464, 'track_err': 0.04749},
-        'gold': {'alpha_std': 0.03312, 'fee_rate': 0.00311, 'track_err': 0.04839}
-
-        # 周因子std
-        self.funcs_weekly = {
-            'hs300': ScoreFunc(alpha=0.3/0.05069, beta=-0.6, fee_rate=-0.1/0.002674),
-            'csi500': ScoreFunc(alpha=0.8/0.081316, beta=-0.2),
-            'gem': ScoreFunc(alpha=0.05/0.060555, beta=-0.50, track_err=-0.45/0.0554576),
-            'national_debt': ScoreFunc(alpha=0.3/0.07519359, beta=-0.25, track_err=-0.45/0.052457),
-            'mmf':ScoreFunc(alpha=0.6/0.0063286, fee_rate=-0.3/0.0008381, track_err=-0.1/0.000555),
-            'sp500rmb': ScoreFunc(fee_rate=-0.2/0.002261574, track_err=-0.8/0.014049778),
-            'gold': ScoreFunc(alpha=0.15/0.01447791, fee_rate=-0/0.0031002, track_err=-0.85/0.02464),
-        }
- 
-        '''
         self.funcs = {
-            'hs300': ScoreFunc(alpha=0.6/ 0.0444, beta=-0.3, track_err=-0.0/0.03939 ,fee_rate=-0.0/ 0.00266),
-            'csi500': ScoreFunc(alpha=0.9/ 0.08291, beta=-0.1),
-            'gem':ScoreFunc(alpha=0.3/0.04332, beta=-0.50, track_err=-0.2/0.04749),
-            'national_debt': ScoreFunc(alpha=0.3/0.08303, beta=-0.5, track_err=-0.4/0.05373, fee_rate=-0.0/0.00229),
-            'mmf': ScoreFunc(alpha=0.3/0.00917, beta=-0.2, fee_rate=-0.1/0.00084, track_err=-0.4/0.00123),
-            'sp500rmb': ScoreFunc(fee_rate=-0.2/0.00228, track_err=-0.8/0.01088),
-            'gold': ScoreFunc(alpha=0.15/0.03312, fee_rate=0, track_err=-0.85/0.04839),
+            'hs300': ScoreFunc(alpha=0.6, beta=-0.3),
+            'csi500': ScoreFunc(alpha=0.9, beta=-0.1),
+            'gem':ScoreFunc(alpha=0.3, beta=-0.50, track_err=-0.2),
+            'national_debt': ScoreFunc(alpha=0.20, beta=-0.45, track_err=-0.35),
+            'mmf': ScoreFunc(alpha=0.3, beta=-0.2, fee_rate=-0.1, track_err=-0.4),
+            'sp500rmb': ScoreFunc(fee_rate=-0.2, track_err=-0.8),
+            'gold': ScoreFunc(alpha=0.15, track_err=-0.85),
             'hsi': ScoreFunc(alpha=0.3, fee_rate=-0.3, track_err=-0.4),# set default score equation
         }
         self.score_cache = defaultdict(dict)
@@ -166,6 +127,10 @@ class FundScoreManager:
         self.score_penalty_param = score_penalty_param
         self._size_and_com_hold_cache: Dict[Any, pd.Index] = {}
 
+    # 获取字符串形式的各大类的默认score func
+    def get_default_func_str(self) -> Dict[str, str]:
+        return {index_id: str(score_func) for index_id, score_func in self.funcs.items()}
+
     def set_param(self, score_param: FundScoreParam):
         self.params = score_param
 
@@ -233,13 +198,17 @@ class FundScoreManager:
         # 上面这一行会耗时10秒左右，且之后遍历index(datetime-index_id pair)也会比下边groupby慢一些
         # 这里应该没必要先groupby datetime，再groupby index_id（如果真这样可能更慢吧）
         self.dts.fund_indicator.groupby(by=['datetime', 'index_id']).apply(lambda x: self._wrapper(x, is_filter_c, use_weekly_monthly_indicators))
-
+        
     def _get_score(self, index_id, dt, cur_d, is_filter_c, use_weekly_monthly_indicators=False, score_func: Optional[Union[ScoreFunc, str]] = None):
         fund_ids = self._filter_size_and_com_hold(dt, index_id, cur_d.index)
         if fund_ids is not cur_d.index:
             # 这里没有dropna总计可以快24秒
             cur_d = cur_d.reindex(fund_ids)
         if cur_d.shape[0] > 1:
+            cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+            cur_d_sta['beta'] = cur_d['beta']
+            cur_d_sta['year_length'] = cur_d['year_length']
+            cur_d = cur_d_sta
             # 不做归一化
             func = self.funcs.get(index_id) if score_func is None else score_func
             if isinstance(func, str):
@@ -256,7 +225,6 @@ class FundScoreManager:
                     score_raw = cur_d.apply(lambda x: func.get_all(x), axis=1)
                 else:
                     score_raw = cur_d.apply(lambda x: func.get(x), axis=1)
-
             if index_id in self.score_penalty_param.FilterYearIndex:
                 punish_funds = cur_d[cur_d.year_length < self.score_penalty_param.JudgeYearLength].index.tolist()
                 for _fund_id in punish_funds:
@@ -318,10 +286,16 @@ class FundScoreManager:
         return score, score_raw
 
     def _white_black_func(self, score, disproved_set):
+        del_list = []
         for index_id in score:
             for fund_id in score[index_id]:
                 if fund_id in disproved_set:
-                    score[index_id][fund_id] += self.score_penalty_param.BlackListPenalty
+                    # 之前是给非白名单 黑名单的基金给分数惩罚，但是基金数量不足时，会交易
+                    # 现在改为强逻辑，直接删掉
+                    #score[index_id][fund_id] += self.score_penalty_param.BlackListPenalty
+                    del_list.append([index_id, fund_id])
+        for del_i in del_list:
+            del score[del_i[0]][del_i[1]]
         return score
 
     @staticmethod
