@@ -165,7 +165,8 @@ class FundPosItem:
 
     fund_id: str            # 基金ID
     index_id: str           # 基金所属的大类资产
-    volume: float             # 基金份额
+    volume: float           # 基金份额 回测下的后复权的份额
+    unit_volume: float      # 基金单位份额 正常交易的份额
     avg_cost: float=None    # 平均成本
     update_time: datetime.date=None # 更新时间
     is_permitted_fund: bool=None  # 在白名单里或者没有白名单 True   不在白名单里或者在黑名单里  False    
@@ -191,6 +192,7 @@ class FundPosItem:
             else: #非清仓卖出 平均成本不变
                 pass
         self.update_time = trade.trade_date
+        self.unit_volume += trade.fund_unit_volume
         assert self.volume >= -1e-8, f'fund backtest vol {self.volume} has negative value: '
         assert self.avg_cost >= -1e-8, f"fund avg cost can not be negative! (fund_id){self.fund_id} (update time){self.update_time}"
         return True
@@ -360,18 +362,20 @@ class FundPosition:
 
     def calc_mv_n_w(self, 
                     fund_navs: dict, # fund_id -> nav(adjusted net value)
-                    index_id=None): # filter: 只计算某个index_id
+                    index_id=None,
+                    is_real_trade: bool=False): # filter: 只计算某个index_id
         mv = 0
         w_dic = {}
         for pos_item in self.funds.values():
-            if index_id and pos_item.index_id != index_id or pos_item.volume == 0:
+            pos_volume = pos_item.volume if not is_real_trade else pos_item.unit_volume
+            if index_id and pos_item.index_id != index_id or pos_volume == 0:
                 continue
             nav = fund_navs.get(pos_item.fund_id, np.nan)
             if np.isnan(nav):
-                print('warning!!!! nav in nan')
+                print(f'warning!!!! nav in nan {pos_item.fund_id}')
             else:
-                mv += nav * pos_item.volume
-            w_dic[pos_item.fund_id] = nav * pos_item.volume
+                mv += nav * pos_volume
+            w_dic[pos_item.fund_id] = nav * pos_volume
             
         if index_id is None:
             mv = self.cash + mv
@@ -391,6 +395,19 @@ class FundPosition:
         vol_dic['cash'] = self.cash
         return AssetPosition(**vol_dic)
 
+    def calc_asset_position_real_trade(self,
+            unit_fund_navs: dict, # fund_id -> nav(unit net value)
+            current_index_prices: AssetPrice,  # index_id -> nav(adjusted net value)
+        ):
+        mv_dic = {}
+        for pos_item in self.funds.values():
+            nav = unit_fund_navs.get(pos_item.fund_id, np.nan)
+            assert not np.isnan(nav), f'fund nav vol {pos_item.fund_id} has negative nav!'
+            mv_dic[pos_item.index_id] = nav * pos_item.unit_volume + mv_dic.get(pos_item.index_id, 0)
+        vol_dic = {f: m / current_index_prices.__dict__.get(f) for f, m in mv_dic.items()}
+        vol_dic['cash'] = self.cash
+        return AssetPosition(**vol_dic)
+
     def get_volume(self, fund_id):
         item = self.funds.get(fund_id)
         if not item:
@@ -401,7 +418,8 @@ class FundPosition:
         return self.index_fund_list.get(index_id, [])
 
     def add(self, fund_id, index_id, volume, avg_cost, update_time, is_permitted_fund):
-        pos_item = FundPosItem(fund_id=fund_id, index_id=index_id, volume=volume, avg_cost=avg_cost, update_time=update_time, is_permitted_fund=is_permitted_fund)
+        #回测下单位份额默认0
+        pos_item = FundPosItem(fund_id=fund_id, index_id=index_id, volume=volume, unit_volume=0, avg_cost=avg_cost, update_time=update_time, is_permitted_fund=is_permitted_fund)
         self.funds[fund_id] = pos_item
         if index_id not in self.index_fund_list:
             self.index_fund_list[index_id] = []
@@ -525,12 +543,12 @@ class ScoreFilter:
 class ScorePenaltyParams:
 
     FilterYearIndex:list = None
-    Penalty:float = -0.5 
+    Penalty:float = - 50
     JudgeYearLength:float = 1.0
     BlackListPenalty:float = - 10000
 
     def __post_init__(self):
-        self.FilterYearIndex = self.FilterYearIndex or ['national_debt', 'csi500']
+        self.FilterYearIndex = self.FilterYearIndex or ['national_debt', 'csi500', 'gem', 'hs300', 'sp500rmb', 'gold']
 
 if __name__ == '__main__':
     a = AssetWeight(hs300=0.1, csi500=0.1, cash=0.8)
