@@ -23,7 +23,7 @@ class SectorInfo(Base):
     sector_id = Column(CHAR(64), primary_key=True)  # 板块ID
     sector_type = Column(Enum(SectorType), nullable=False)  # 板块类型 industry-行业 topic-主题
     sector_desc = Column(TEXT, nullable=False)  # 板块描述
-    sector_name = Column(CHAR(16), nullable=False)  # 板块名称
+    sector_name = Column(CHAR(16), nullable=False, unique=True)  # 板块名称
     main_index_id = Column(CHAR(16), nullable=False)  # 板块代表指数
 
 
@@ -46,6 +46,7 @@ class IndexInfo(Base):
 
     __tablename__ = 'index_info'
     index_id = Column(CHAR(20), primary_key=True)
+    is_deleted = Column(BOOLEAN, default=False)  # 是否删除
 
     order_book_id = Column(CHAR(20))  # 米筐ID
     # web_id= Column(CHAR(20)) # 数据所在网页ID
@@ -57,7 +58,7 @@ class IndexInfo(Base):
     is_select = Column(BOOLEAN, nullable=False)  # 是否属于精选
     em_id = Column(CHAR(20))  # ChoiceID
     em_plate_id = Column(CHAR(20))  # Choice板块ID
-    index_profile = Column(TEXT, nullable=False)  # 指数概况
+    index_profile = Column(TEXT)  # 指数概况
     price_source = Column(Enum(IndexPriceSource), nullable=False)  # 指数价格来源标记
 
 
@@ -92,6 +93,22 @@ class FundSize(Base):
     latest_size = Column(DOUBLE(asdecimal=False)) # 最新规模
 
 
+class FundStatusLatest(Base):
+    '''
+    基金状态表 包含基金申赎状态等
+    '''
+
+    __tablename__ = 'fund_status_latest'
+
+    fund_id = Column(CHAR(10), primary_key=True)  # 基金ID
+
+    PURCHSTATUS = Column('purch_status', CHAR(16))  # 申购状态
+    REDEMSTATUS = Column('redem_status', CHAR(16))  # 赎回状态
+    LPLIMIT = Column('lp_limit', DOUBLE(asdecimal=False))  # 暂停大额申购上限
+    OTCLPLIMIT = Column('otc_lp_limit', DOUBLE(asdecimal=False))  # 场外暂停大额申购上限
+    TRADESTATUS = Column('trade_status', CHAR(16))  # 交易状态
+
+
 class FundInfo(Base):
     '''基金信息表'''
 
@@ -139,6 +156,10 @@ class FundInfo(Base):
     purchase_fee_detail = Column(TEXT) # 申购费率详情
     redeem_fee_detail = Column(TEXT) # 赎回费率详情
     service_fee = Column(TEXT) # 销售服务费率
+    fund_manager = Column(TEXT)  # 基金经理(现任)
+    pre_fund_manager = Column(TEXT)  # 基金经理(历任)
+    custodian_bank = Column(CHAR(64))  # 基金托管人
+    foreign_custodian = Column(CHAR(64))  # 境外托管人
 
 
 class FundBenchmark(Base):
@@ -152,6 +173,7 @@ class FundBenchmark(Base):
     index_text = Column(TEXT, nullable=False)  # 业绩比较基准
     benchmark = Column(TEXT, nullable=False)  # 解析出来的业绩比较基准
     benchmark_s = Column(TEXT, nullable=False)  # index转换后的业绩比较基准
+    benchmark_s_raw = Column(TEXT, nullable=False)  # index转换后的业绩比较基准(原始)
     assets = Column(CHAR(32))  # 追踪的标的
     industry = Column(CHAR(64))  # 行业分类
 
@@ -189,22 +211,6 @@ class FundNav(Base):
     )
 
 
-class FundEstimatedNav(Base):
-    '''基金估算净值表'''
-
-    __tablename__ = 'fund_estimated_nav'
-    fund_id = Column(CHAR(20), primary_key=True) # 合约代码
-    datetime = Column(DATETIME, primary_key=True) # 估算时间
-
-    unit_net_value = Column(DOUBLE(asdecimal=False)) # 单位净值
-    acc_net_value = Column(DOUBLE(asdecimal=False)) # 累计单位净值 基金公司公告发布的原始数据 单位净值 + 单位累计分红
-    adjusted_net_value = Column(DOUBLE(asdecimal=False)) # 复权净值 考虑分红再投资后调整的单位净值
-
-    __table_args__ = (
-        Index('idx_fund_estimated_nav_datetime', 'datetime'),
-    )
-
-
 class IndexPrice(Base):
     '''指数价格表'''
 
@@ -307,6 +313,27 @@ class StyleAnalysisStockFactor(Base):
         Index('idx_style_analysis_stock_factor_datetime', 'datetime'),
     )
 
+class BarraCNE5RiskFactor(Base):
+    '''Barra CNE5风险因子'''
+
+    __tablename__ = 'barra_cne5_risk_factor'
+
+    stock_id = Column(CHAR(10), primary_key=True)  # EM股票ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    ret = Column(DOUBLE(asdecimal=False))
+    leverage = Column(DOUBLE(asdecimal=False))
+    logsize = Column(DOUBLE(asdecimal=False))
+    momentum = Column(DOUBLE(asdecimal=False))
+    book_price = Column(DOUBLE(asdecimal=False))
+    earning_yields = Column(DOUBLE(asdecimal=False))
+    liquidity = Column(DOUBLE(asdecimal=False))
+    growth = Column(DOUBLE(asdecimal=False))
+    resvol = Column(DOUBLE(asdecimal=False))
+
+    __table_args__ = (
+        Index('idx_barra_cne5_risk_factor_datetime', 'datetime'),
+    )
+
 class Fund_size_and_hold_rate(Base):
     '''基金规模和持有人比例'''
 
@@ -317,6 +344,7 @@ class Fund_size_and_hold_rate(Base):
     size = Column(DOUBLE(asdecimal=False)) # 最新规模
     personal_holds = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比
     institution_holds = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比
+    hold_num = Column(DOUBLE(asdecimal=False))  # 基金份额持有人户数
 
 class FundRate(Base):
     '''基金评级'''
@@ -359,6 +387,8 @@ class FundHoldAsset(Base):
     bond_nav_ratio = Column(DOUBLE(asdecimal=False)) # 债券市值占基金资产净值比
     cash_nav_ratio = Column(DOUBLE(asdecimal=False)) # 银行存款占基金资产净值比
     other_nav_ratio = Column(DOUBLE(asdecimal=False)) # 其他资产占基金资产净值比
+    first_repo_to_nav = Column(DOUBLE(asdecimal=False))  # 报告期内债券回购融资余额占基金资产净值比例
+    avg_ptm = Column(DOUBLE(asdecimal=False))  # 报告期末投资组合平均剩余期限
 
 class FundHoldIndustry(Base):
     '''基金持仓前三行业'''
@@ -467,4 +497,92 @@ class FundHoldBond(Base):
     rank10_bond = Column(CHAR(30)) #债券名
     rank10_bond_code =  Column(CHAR(15)) # 债券id
     rank10_bondval = Column(DOUBLE(asdecimal=False)) #持仓市值
-    rank10_bondweight = Column(DOUBLE(asdecimal=False)) #持仓权重
\ No newline at end of file
+    rank10_bondweight = Column(DOUBLE(asdecimal=False)) #持仓权重
+
+class MngInfo(Base):
+    '''基金经理信息表'''
+
+    __tablename__ = 'mng_info'
+
+    start_mng_date = Column(DATE, primary_key=True) # 最早任职基金经理日期
+    mng_name = Column(CHAR(25), primary_key=True) # 基金经理
+    fund_id = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    fund_size = Column(DOUBLE(asdecimal=False)) # 在管基金规模(亿元)
+    start_date = Column(DATE) # 任职日期
+    work_days = Column(DOUBLE(asdecimal=False)) # 任职天数
+    his_ret = Column(DOUBLE(asdecimal=False)) # 任职以来回报(%)
+    his_annual_ret = Column(DOUBLE(asdecimal=False)) # 任职以来年化回报(%)
+    benchmark = Column(TEXT) # 业绩比较基准
+    fund_type = Column(CHAR(25)) # 基金类型
+    company_name = Column(CHAR(25)) # 管理公司
+    coworkers = Column(CHAR(40)) # 共同任职经理
+    mng_work_year = Column(DOUBLE(asdecimal=False)) # 基金经理年限
+    worked_funds_num = Column(DOUBLE(asdecimal=False)) # 任职基金数
+    worked_com_num = Column(DOUBLE(asdecimal=False)) # 任职基金公司数
+    fund_total_geo_ret = Column(DOUBLE(asdecimal=False)) # 任职基金几何总回报(%)
+    geo_annual_ret = Column(DOUBLE(asdecimal=False)) # 几何平均年化收益率(%)
+    cal_annual_ret = Column(DOUBLE(asdecimal=False)) # 算术平均年化收益率(%)
+    gender = Column(CHAR(2)) # 性别
+    birth_year = Column(CHAR(4)) # 出生年份
+    age =  Column(DOUBLE(asdecimal=False)) # 年龄
+    education = Column(CHAR(4)) # 学历
+    nationality =  Column(CHAR(10)) # 国籍
+    resume = Column(TEXT) # 简历
+    
+class FundMngChange(Base):
+    '''基金经理变更表'''
+
+    __tablename__ = 'fund_mng_change'
+
+    fund_id = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    mng_now = Column(CHAR(40)) # 现任基金经理
+    mng_now_start_date = Column(TEXT) # 现任经理最新任职日期
+    mng_begin = Column(CHAR(40)) # 最早任职基金经理
+    mng_begin_date = Column(DATE) # 最早任职基金经理日期
+    mng_now_work_year = Column(DOUBLE(asdecimal=False)) # 现任基金经理年限
+    same_type_fund_work_year = Column(DOUBLE(asdecimal=False)) # 同类型基金现任基金经理年限均值
+    resign_mngs = Column(TEXT) # 已离任基金经理
+    total_mng_num = Column(DOUBLE(asdecimal=False)) # 已离任基金经理
+    total_mng_avg_work_year = Column(DOUBLE(asdecimal=False)) # 历任基金经理人均任职年限
+    fund_type = Column(CHAR(25)) # 基金类型
+    company = Column(CHAR(35)) # 管理公司
+    region = Column(CHAR(5)) # 监管辖区
+
+class FundCompMngChange(Base):
+    '''基金公司经理变更表'''
+
+    __tablename__ = 'fund_comp_mng_change'
+
+    company = Column(CHAR(35),  primary_key=True ) # 管理公司
+    total_mng_num = Column(DOUBLE(asdecimal=False)) # 基金经理数
+    mng_avg_year = Column(DOUBLE(asdecimal=False)) # 基金经理平均年限
+    mng_max_year = Column(DOUBLE(asdecimal=False)) # 基金经理最大年限
+    team_stability = Column(DOUBLE(asdecimal=False)) # 团队稳定性
+    new_mng_num = Column(DOUBLE(asdecimal=False)) # 新聘基金经理数
+    resign_mng_num = Column(DOUBLE(asdecimal=False)) # 离职基金经理数
+    mng_turnover_rate = Column(DOUBLE(asdecimal=False)) # 基金经理变动率
+    exp_less_than_1 = Column(DOUBLE(asdecimal=False)) # 1年以内
+    exp_1_to_2 = Column(DOUBLE(asdecimal=False)) # 1-2年
+    exp_2_to_3 = Column(DOUBLE(asdecimal=False)) # 2-3年
+    exp_3_to_4 = Column(DOUBLE(asdecimal=False)) # 3-4年
+    exp_more_than_4 = Column(DOUBLE(asdecimal=False)) # 4年以上
+    
+class FundCompCoreMng(Base):
+    '''基金公司核心人员稳定性'''
+
+    __tablename__ = 'fund_comp_core_mng'
+
+    fund_id = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    fund_resign_mng_num = Column(DOUBLE(asdecimal=False)) # 本基金离职基金经理人数
+    company = Column(CHAR(35)) # 管理公司
+    com_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司基金经理数	
+    com_resign_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司离职基金经理数
+    com_resign_mng_rate = Column(DOUBLE(asdecimal=False)) # 基金公司基金经理离职率(%)	
+    com_core_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司核心人员人数
+    com_core_mng_resign_num = Column(DOUBLE(asdecimal=False)) #基金公司核心人员离职人数
+    com_core_mng_resign_rate = 	Column(DOUBLE(asdecimal=False)) #基金公司核心人员离职率(%)
+    fund_type = Column(CHAR(25)) # 基金类型
+    region = Column(CHAR(5)) # 监管辖区
\ No newline at end of file
