@@ -109,6 +109,31 @@ class IndexValuation(Base):
         Index('idx_index_valuation_develop_datetime', 'datetime'),
     )
 
+class IndexValuationLongTerm(Base):
+    '''指数估值'''
+
+    __tablename__ = 'index_valuation_long_term'
+    index_id = Column(CHAR(20), primary_key=True) #指数ID
+    datetime = Column(DATE, primary_key=True) # 日期
+
+    pb_mrq = Column(DOUBLE(asdecimal=False)) # 市净率-MRQ
+    pe_ttm = Column(DOUBLE(asdecimal=False)) # 市盈率-MMT
+    roe = Column(DOUBLE(asdecimal=False)) # 净资产收益率
+    ps_ttm = Column(DOUBLE(asdecimal=False)) # 市销率—MMT
+    dy = Column(DOUBLE(asdecimal=False)) # 股息率
+    pcf_ttm = Column(DOUBLE(asdecimal=False)) # 市现率-MMT
+    peg_ttm = Column(DOUBLE(asdecimal=False)) # 市盈率相对盈利增长比率-MMT
+    pe_pct = Column(DOUBLE(asdecimal=False)) # PE百分位
+    pb_pct = Column(DOUBLE(asdecimal=False)) # PB百分位
+    ps_pct = Column(DOUBLE(asdecimal=False)) # PS百分位
+    val_score = Column(DOUBLE(asdecimal=False)) # 估值评分
+    eps_ttm = Column(DOUBLE(asdecimal=False)) # 每股收益—MMT
+    roe_pct = Column(DOUBLE(asdecimal=False)) # ROE百分位
+    est_peg = Column(DOUBLE(asdecimal=False)) # 预测peg
+
+    __table_args__ = (
+        Index('idx_index_valuation_long_term_datetime', 'datetime'),
+    )
 
 class FundIndicator(Base):
     '''基金评价指标'''
@@ -190,6 +215,52 @@ class FundIndicatorAnnual(Base):
     )
 
 
+class FundIndicatorGroup(Base):
+    '''基金评价指标'''
+
+    __tablename__ = 'fund_indicator_group'
+
+    fund_id = Column(CHAR(10), primary_key=True)  # 基金ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    scale = Column(DOUBLE(asdecimal=False))  # 规模
+    beta = Column(DOUBLE(asdecimal=False))
+    alpha = Column(DOUBLE(asdecimal=False))
+    track_err = Column(DOUBLE(asdecimal=False))  # 跟踪误差
+    info_ratio = Column(DOUBLE(asdecimal=False))  # 信息比率
+    treynor = Column(DOUBLE(asdecimal=False))  # 特雷诺比率
+    mdd = Column(DOUBLE(asdecimal=False))  # 净值最大回撤
+    annual_vol = Column(DOUBLE(asdecimal=False))  # 年化波动率
+    annual_ret = Column(DOUBLE(asdecimal=False))  # 年化收益
+    sharpe = Column(DOUBLE(asdecimal=False))
+    calma_ratio = Column(DOUBLE(asdecimal=False))
+    stutzer = Column(DOUBLE(asdecimal=False))
+    continue_regress_v = Column(DOUBLE(asdecimal=False))  # 业绩持续性回归系数
+    continue_regress_t = Column(DOUBLE(asdecimal=False))  # 业绩持续性回归T值
+    continue_stats_v = Column(DOUBLE(asdecimal=False))  # 业绩持续性回归计算值
+    stock_cl_alpha = Column(DOUBLE(asdecimal=False))  # cl alpha
+    stock_cl_alpha_t = Column(DOUBLE(asdecimal=False))  # cl alpha t值
+    stock_cl_alpha_p = Column(DOUBLE(asdecimal=False))  # cl alpha p值
+    stock_cl_beta = Column(DOUBLE(asdecimal=False))  # cl beta
+    stock_cl_beta_t = Column(DOUBLE(asdecimal=False))  # cl beta t值
+    stock_cl_beta_p = Column(DOUBLE(asdecimal=False))  # cl beta p值
+    stock_tm_alpha = Column(DOUBLE(asdecimal=False))  # tm alpha
+    stock_tm_alpha_t = Column(DOUBLE(asdecimal=False))  # tm alpha t值
+    stock_tm_alpha_p = Column(DOUBLE(asdecimal=False))  # tm alpha p值
+    stock_tm_beta = Column(DOUBLE(asdecimal=False))  # tm beta
+    stock_tm_beta_t = Column(DOUBLE(asdecimal=False))  # tm beta t值
+    stock_tm_beta_p = Column(DOUBLE(asdecimal=False))  # tm beta p值
+    stock_hm_alpha = Column(DOUBLE(asdecimal=False))  # hm alpha
+    stock_hm_alpha_t = Column(DOUBLE(asdecimal=False))  # hm alpha t值
+    stock_hm_alpha_p = Column(DOUBLE(asdecimal=False))  # hm alpha p值
+    stock_hm_beta = Column(DOUBLE(asdecimal=False))  # hm beta
+    stock_hm_beta_t = Column(DOUBLE(asdecimal=False))  # hm beta t值
+    stock_hm_beta_p = Column(DOUBLE(asdecimal=False))  # hm beta p值
+
+    __table_args__ = (
+        Index('idx_fund_indicator_group_datetime', 'datetime'),
+    )
+
+
 class FundScore(Base):
     '''基金评分'''
 
@@ -218,28 +289,36 @@ class FundScoreExtended(Base):
 
     fund_id = Column(CHAR(16), primary_key=True)  # 基金代码
     datetime = Column(DATE, primary_key=True)  # 日期
-    wind_class_1 = Column(CHAR(64))  # Wind基金类型
-    ret = Column(DOUBLE(asdecimal=False), nullable=False)  # ret
-    vol = Column(DOUBLE(asdecimal=False), nullable=False)  # volatility
+    wind_class_1 = Column(CHAR(16))  # Wind基金一级类型
+    ret = Column(DOUBLE(asdecimal=False))  # ret
+    vol = Column(DOUBLE(asdecimal=False))  # volatility
     sr = Column(DOUBLE(asdecimal=False))  # sharpe ratio
     ir = Column(DOUBLE(asdecimal=False))  # information ratio
     beta = Column(DOUBLE(asdecimal=False))  # beta
     alpha = Column(DOUBLE(asdecimal=False))  # alpha
-    mdd = Column(DOUBLE(asdecimal=False), nullable=False)  # max drawdown
+    treynor_ratio = Column(DOUBLE(asdecimal=False))  # treynor_ratio
+    mdd = Column(DOUBLE(asdecimal=False))  # max drawdown
     timing = Column(DOUBLE(asdecimal=False))  # timing
-    ret_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of ret
-    vol_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of volatility
+    ret_score = Column(DOUBLE(asdecimal=False))  # score of ret
+    vol_score = Column(DOUBLE(asdecimal=False))  # score of volatility
     sr_score = Column(DOUBLE(asdecimal=False))  # score of sharpe ratio
     ir_score = Column(DOUBLE(asdecimal=False))  # score of information ratio
     beta_score = Column(DOUBLE(asdecimal=False))  # score of beta
     alpha_score = Column(DOUBLE(asdecimal=False))  # score of alpha
-    mdd_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of max drawdown
-    return_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of return
-    robust_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of robust
+    treynor_ratio_score = Column(DOUBLE(asdecimal=False))  # score of treynor_ratio
+    mdd_score = Column(DOUBLE(asdecimal=False))  # score of max drawdown
+    return_score = Column(DOUBLE(asdecimal=False))  # score of return
+    robust_score = Column(DOUBLE(asdecimal=False))  # score of robust
     timing_score = Column(DOUBLE(asdecimal=False))  # score of timing
-    return_rank = Column(DOUBLE(asdecimal=False), nullable=False)  # rank of return
-    robust_rank = Column(DOUBLE(asdecimal=False), nullable=False)  # rank of robust
+    selection_score = Column(DOUBLE(asdecimal=False))  # score of selection
+    allocation_score = Column(DOUBLE(asdecimal=False))  # score of allocation
+    interaction_score = Column(DOUBLE(asdecimal=False))  # score of interaction
+    return_rank = Column(DOUBLE(asdecimal=False))  # rank of return
+    robust_rank = Column(DOUBLE(asdecimal=False))  # rank of robust
     timing_rank = Column(DOUBLE(asdecimal=False))  # rank of timing
+    selection_rank = Column(DOUBLE(asdecimal=False))  # rank of selection
+    allocation_rank = Column(DOUBLE(asdecimal=False))  # rank of allocation
+    interaction_rank = Column(DOUBLE(asdecimal=False))  # rank of interaction
     total_score = Column(DOUBLE(asdecimal=False), nullable=False)  # score of total
     total_rank = Column(DOUBLE(asdecimal=False), nullable=False)  # rank of total
 
@@ -291,6 +370,22 @@ class StyleAnalysisFactorReturn(Base):
     high_low = Column(DOUBLE(asdecimal=False))  # 波动率
     const = Column(DOUBLE(asdecimal=False))  # 常数项
 
+class BarraCNE5FactorReturn(Base):
+    '''Barra CNE5因子收益'''
+
+    __tablename__ = 'barra_cne5_factor_return'
+
+    datetime = Column(DATE, primary_key=True)  # 日期
+    market = Column(DOUBLE(asdecimal=False))
+    leverage = Column(DOUBLE(asdecimal=False))
+    cap = Column(DOUBLE(asdecimal=False))
+    momentum = Column(DOUBLE(asdecimal=False))
+    bp = Column(DOUBLE(asdecimal=False))
+    earning = Column(DOUBLE(asdecimal=False))
+    liquidity = Column(DOUBLE(asdecimal=False))
+    growth = Column(DOUBLE(asdecimal=False))
+    vol = Column(DOUBLE(asdecimal=False))
+
 class AllocationDistribution(Base):
     '''有效前沿下各mdd最优策略'''
 
@@ -324,3 +419,26 @@ class FundAIC(Base):
     intel_y3_ret = Column(DOUBLE(asdecimal=False))  # 智能定投三年收益
     intel_y5_ret = Column(DOUBLE(asdecimal=False))  # 智能定投五年收益
 
+class StyleBox(Base):
+    '''基金风格九宫格评分'''
+
+    __tablename__ = 'fund_style_box'
+    fund_id = Column(CHAR(20), primary_key=True)  # 基金ID
+    datetime = Column(CHAR(20), primary_key=True)  # 时间
+    x = Column(DOUBLE(asdecimal=False))  # 价值-平衡-成长维度
+    y = Column(DOUBLE(asdecimal=False))  # 小盘-中盘-大盘维度
+
+
+class StyleReg(Base):
+    '''Style 因子收益'''
+
+    __tablename__ = 'style_reg_factor_return'
+
+    datetime = Column(DATE, primary_key=True)  # 日期
+    large_g= Column(DOUBLE(asdecimal=False))
+    large_v = Column(DOUBLE(asdecimal=False))
+    mid_g = Column(DOUBLE(asdecimal=False))
+    mid_v = Column(DOUBLE(asdecimal=False))
+    small_g = Column(DOUBLE(asdecimal=False))
+    small_v = Column(DOUBLE(asdecimal=False))
+   
