@@ -102,6 +102,26 @@ class RqStockMinute(Base):
     )
 
 
+class RqIndexMinute(Base):
+    '''米筐指数分钟线数据表'''
+
+    __tablename__ = 'rq_index_minute'
+
+    order_book_id = Column(CHAR(20), primary_key=True) # 指数ID
+    datetime = Column(DATETIME, primary_key=True) # 日期
+
+    open = Column(DOUBLE(asdecimal=False)) # 开盘价
+    close = Column(DOUBLE(asdecimal=False)) # 收盘价
+    high = Column(DOUBLE(asdecimal=False)) # 最高价
+    low = Column(DOUBLE(asdecimal=False)) # 最低价
+    total_turnover = Column(DOUBLE(asdecimal=False)) # 成交额
+    volume = Column(DOUBLE(asdecimal=False)) # 成交量  不复权和后复权的交易量不同
+
+    __table_args__ = (
+        Index('idx_rq_index_minute_datetime', 'datetime'),
+    )
+
+
 class RqFundNav(Base):
     '''米筐基金净值表'''
 
@@ -238,6 +258,21 @@ class RqIndexComponent(Base):
     stock_list = Column(TEXT) # 股票列表
 
 
+class SinaStockMinute(Base):
+    '''米筐股票分钟线数据表'''
+
+    __tablename__ = 'sina_stock_minute'
+
+    stock_id = Column(CHAR(20), primary_key=True) # 股票ID
+    datetime = Column(DATETIME, primary_key=True) # 日期
+
+    close = Column(DOUBLE(asdecimal=False)) # 收盘价
+
+    __table_args__ = (
+        Index('idx_sina_stock_minute_datetime', 'datetime'),
+    )
+
+
 class EmIndexPrice(Base):
     '''Choice指数数据'''
 
@@ -246,6 +281,16 @@ class EmIndexPrice(Base):
     datetime = Column(DATE, primary_key=True) # 日期
     close = Column(DOUBLE(asdecimal=False)) # 收盘价
 
+
+class EmRealtimeIndexPrice(Base):
+    '''Choice实时指数数据'''
+
+    __tablename__ = 'em_realtime_index_price'
+    em_id = Column(CHAR(20), primary_key=True) # Choice代码
+    datetime = Column(DATETIME, primary_key=True) # 数据时间
+    price = Column(DOUBLE(asdecimal=False)) # 指数点位
+
+
 class EmIndexInfo(Base):
     '''Choice指数信息'''
 
@@ -254,10 +299,10 @@ class EmIndexInfo(Base):
     CODES = Column('em_id', CHAR(20), primary_key=True)  # Choice代码
 
     NAME = Column('name', CHAR(64), nullable=False)  # 指数名称
-    SHORTNAME = Column('short_name', CHAR(32), nullable=False)  # 指数简称
+    SHORTNAME = Column('short_name', CHAR(64), nullable=False)  # 指数简称
     PUBLISHDATE = Column('publish_date', DATE)  # 发布日期
     MAKERNAME = Column('maker_name', CHAR(32), nullable=False)  # 编制机构
-    INDEXPROFILE = Column('index_profile', TEXT, nullable=False)  # 指数概况
+    INDEXPROFILE = Column('index_profile', TEXT)  # 指数概况
     price_source = Column(Enum(IndexPriceSource), nullable=False)  # 指数价格来源标记
 
 class FundFee(Base):
@@ -629,6 +674,8 @@ class EmStockFinFac(Base):
     EPSBASIC = Column('eps_basic', DOUBLE(asdecimal=False)) # 每股收益EPS(基本)
     EPSDILUTED = Column('eps_diluted', DOUBLE(asdecimal=False)) # 每股收益EPS(稀释)
     BPS = Column('bps', DOUBLE(asdecimal=False)) # 每股净资产
+    MBREVENUE = Column('mb_revenue', DOUBLE(asdecimal=False)) # 主营业务收入
+    MBCOST = Column('mb_cost', DOUBLE(asdecimal=False)) # 主营业务支出
     BALANCESTATEMENT_25 = Column('balance_statement_25', DOUBLE(asdecimal=False)) # 流动资产合计
     BALANCESTATEMENT_46 = Column('balance_statement_46', DOUBLE(asdecimal=False)) # 非流动资产合计
     BALANCESTATEMENT_74 = Column('balance_statement_74', DOUBLE(asdecimal=False)) # 资产总计
@@ -702,7 +749,7 @@ class EmIndexVal(Base):
     PCFTTM = Column("pcf_ttm", DOUBLE(asdecimal=False)) # 市现率PCF(TTM)
     ROE = Column("roe", DOUBLE(asdecimal=False)) # 净资产收益率
     EPSTTM = Column("eps_ttm", DOUBLE(asdecimal=False)) # 每股收益TTM
-    ESTPEG = Column("est_peg",DOUBLE(asdecimal=False)) # 预测PEG
+    ESTPEG = Column("est_peg", DOUBLE(asdecimal=False)) # 预测PEG
 
 class EmTradeDates(Base):
     '''Choice交易日数据'''
@@ -736,12 +783,14 @@ class CSIndexComponent(Base):
 
 class EmFundHoldingRate(Base):
     '''Choice基金持有人比例'''
+
     __tablename__ = 'em_fund_holding_rate'
 
     CODES = Column(CHAR(10), primary_key=True) # 合约代码
     DATES = Column(DATE, primary_key=True) # 日期
     HOLDPERSONALHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比
     HOLDINSTIHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比
+    HOLDNUM = Column(DOUBLE(asdecimal=False))  # 基金份额持有人户数
 
 class EmFundList(Base):
     '''Choice基金列表'''
@@ -770,6 +819,8 @@ class EmFundInfo(Base):
     FUNDTYPE = Column('is_open', BOOLEAN, nullable=False) # 是否定期开放基金, 以Choice的FUNDTYPE来判断
     PREDFUNDMANAGER = Column('manager_id', TEXT, nullable=False) # 基金经理(历任)
     MGRCOMP = Column('company_id', CHAR(64), nullable=False) # 基金公司名称
+    CUSTODIANBANK = Column('custodian_bank', CHAR(64))  # 基金托管人
+    FOREIGNCUSTODIAN = Column('foreign_custodian', CHAR(64))  # 境外托管人
 
 class EmFundBenchmark(Base):
     '''Choice基金业绩比较基准表'''
@@ -958,6 +1009,8 @@ class EMFundHoldAsset(Base):
     PRTBONDTONAV = Column(DOUBLE(asdecimal=False))#债券市值占基金资产净值比
     PRTCASHTONAV = Column(DOUBLE(asdecimal=False))#银行存款占基金资产净值比
     PRTOTHERTONAV = Column(DOUBLE(asdecimal=False))#其他资产占基金资产净值比
+    MMFFIRSTREPOTONAV = Column(DOUBLE(asdecimal=False))  # 报告期内债券回购融资余额占基金资产净值比例
+    MMFAVGPTM = Column(DOUBLE(asdecimal=False))  # 报告期末投资组合平均剩余期限
 
 class EMFundHoldIndustry(Base):
     '''东财基金行业配置'''
@@ -1067,3 +1120,124 @@ class EMFundHoldBond(Base):
     rank10_bond_code =  Column(CHAR(15)) # 债券id
     rank10_bondval = Column(DOUBLE(asdecimal=False)) #持仓市值
     rank10_bondweight = Column(DOUBLE(asdecimal=False)) #持仓权重
+
+class EmStockDividend(Base):
+    '''东财股票分红数据 '''
+
+    __tablename__ = 'em_stock_dividend'
+
+    CODES = Column(CHAR(10), primary_key=True) # 东财基金ID
+    REPORT_DATES = Column(DATE, primary_key=True) # 报告期
+    DATES = Column(DATE, primary_key=True) # 日期
+    DIVCASHPSBFTAX = Column(DOUBLE(asdecimal=False)) # 每股股利(税前)
+    DIVSTOCKPS = Column(DOUBLE(asdecimal=False)) # 每股红股
+    DIVPROGRESS = Column(CHAR(16)) # 分红方案进度
+    DIVRECORDDATE = Column(DATE) # 股权登记日
+    DIVEXDATE = Column(DATE) # 除权除息日
+    DIVPRENOTICEDATE = Column(DATE) # 预披露公告日
+    DIVIMPLANNCDATE = Column(DATE) # 分红实施公告日
+    DIVAGMANNCDATE = Column(DATE) # 股东大会公告日
+    DIVPLANANNCDATE = Column(DATE) # 预案公告日
+    DIVORNOT = Column(BOOLEAN) # 是否分红
+
+class EMMngInfo(Base):
+    '''东财基金经理信息表'''
+
+    __tablename__ = 'em_mng_info'
+
+    start_mng_date = Column(DATE, primary_key=True) # 最早任职基金经理日期
+    mng_name = Column(CHAR(25), primary_key=True) # 基金经理
+    code = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    fund_size = Column(DOUBLE(asdecimal=False)) # 在管基金规模(亿元)
+    start_date = Column(DATE) # 任职日期
+    work_days = Column(DOUBLE(asdecimal=False)) # 任职天数
+    his_ret = Column(DOUBLE(asdecimal=False)) # 任职以来回报(%)
+    his_annual_ret = Column(DOUBLE(asdecimal=False)) # 任职以来年化回报(%)
+    benchmark = Column(TEXT) # 业绩比较基准
+    fund_type = Column(CHAR(25)) # 基金类型
+    company_name = Column(CHAR(25)) # 管理公司
+    coworkers = Column(CHAR(40)) # 共同任职经理
+    mng_work_year = Column(DOUBLE(asdecimal=False)) # 基金经理年限
+    worked_funds_num = Column(DOUBLE(asdecimal=False)) # 任职基金数
+    worked_com_num = Column(DOUBLE(asdecimal=False)) # 任职基金公司数
+    fund_total_geo_ret = Column(DOUBLE(asdecimal=False)) # 任职基金几何总回报(%)
+    geo_annual_ret = Column(DOUBLE(asdecimal=False)) # 几何平均年化收益率(%)
+    cal_annual_ret = Column(DOUBLE(asdecimal=False)) # 算术平均年化收益率(%)
+    gender = Column(CHAR(2)) # 性别
+    birth_year = Column(CHAR(4)) # 出生年份
+    age =  Column(DOUBLE(asdecimal=False)) # 年龄
+    education = Column(CHAR(4)) # 学历
+    nationality =  Column(CHAR(10)) # 国籍
+    resume = Column(TEXT) # 简历
+
+class EMFundMngChange(Base):
+    '''东财基金经理变更表'''
+
+    __tablename__ = 'em_fund_mng_change'
+
+    code = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    mng_now = Column(CHAR(40)) # 现任基金经理
+    mng_now_start_date = Column(TEXT) # 现任经理最新任职日期
+    mng_begin = Column(CHAR(40)) # 最早任职基金经理
+    mng_begin_date = Column(DATE) # 最早任职基金经理日期
+    mng_now_work_year = Column(DOUBLE(asdecimal=False)) # 现任基金经理年限
+    same_type_fund_work_year = Column(DOUBLE(asdecimal=False)) # 同类型基金现任基金经理年限均值
+    resign_mngs = Column(TEXT) # 已离任基金经理
+    total_mng_num = Column(DOUBLE(asdecimal=False)) # 已离任基金经理
+    total_mng_avg_work_year = Column(DOUBLE(asdecimal=False)) # 历任基金经理人均任职年限
+    fund_type = Column(CHAR(25)) # 基金类型
+    company = Column(CHAR(35)) # 管理公司
+    region = Column(CHAR(5)) # 监管辖区
+
+class EMFundCompMngChange(Base):
+    '''东财基金公司经理变更表'''
+
+    __tablename__ = 'em_fund_comp_mng_change'
+
+    company = Column(CHAR(35),  primary_key=True ) # 管理公司
+    total_mng_num = Column(DOUBLE(asdecimal=False)) # 基金经理数
+    mng_avg_year = Column(DOUBLE(asdecimal=False)) # 基金经理平均年限
+    mng_max_year = Column(DOUBLE(asdecimal=False)) # 基金经理最大年限
+    team_stability = Column(DOUBLE(asdecimal=False)) # 团队稳定性
+    new_mng_num = Column(DOUBLE(asdecimal=False)) # 新聘基金经理数
+    resign_mng_num = Column(DOUBLE(asdecimal=False)) # 离职基金经理数
+    mng_turnover_rate = Column(DOUBLE(asdecimal=False)) # 基金经理变动率
+    exp_less_than_1 = Column(DOUBLE(asdecimal=False)) # 1年以内
+    exp_1_to_2 = Column(DOUBLE(asdecimal=False)) # 1-2年
+    exp_2_to_3 = Column(DOUBLE(asdecimal=False)) # 2-3年
+    exp_3_to_4 = Column(DOUBLE(asdecimal=False)) # 3-4年
+    exp_more_than_4 = Column(DOUBLE(asdecimal=False)) # 4年以上
+
+class EMFundCompCoreMng(Base):
+    '''东财基金公司核心人员稳定性'''
+
+    __tablename__ = 'em_fund_comp_core_mng'
+
+    code = Column(CHAR(10), primary_key=True) # 代码
+    desc_name = Column(CHAR(70)) # 名称
+    fund_resign_mng_num = Column(DOUBLE(asdecimal=False)) # 本基金离职基金经理人数
+    company = Column(CHAR(35)) # 管理公司
+    com_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司基金经理数
+    com_resign_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司离职基金经理数
+    com_resign_mng_rate = Column(DOUBLE(asdecimal=False)) # 基金公司基金经理离职率(%)
+    com_core_mng_num = Column(DOUBLE(asdecimal=False)) # 基金公司核心人员人数
+    com_core_mng_resign_num = Column(DOUBLE(asdecimal=False)) #基金公司核心人员离职人数
+    com_core_mng_resign_rate = 	Column(DOUBLE(asdecimal=False)) #基金公司核心人员离职率(%)
+    fund_type = Column(CHAR(25)) # 基金类型
+    region = Column(CHAR(5)) # 监管辖区
+
+class EmBondRating(Base):
+    '''Choice债券评级'''
+
+    __tablename__ = 'em_bond_rating'
+
+    CODES = Column('bond_id', CHAR(16), primary_key=True) # EM债券ID
+    DATES = Column('datetime', DATE, primary_key=True) # 日期
+
+    APPORATE = Column('appo_rate', CHAR(8))  # 指定日债项评级
+
+    __table_args__ = (
+        Index('idx_em_bond_rating_datetime', 'datetime'),
+    )
