@@ -97,7 +97,7 @@ class FundEngine:
         self._prep_target_asset_allocation = self.calc_asset_allocation(dt)
         self._prep_target_fund_allocation = self.calc_fund_allocation(dt) if self.is_fund_backtest else None
 
-    def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
+    def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None, is_real_trade=False):
         if not self.is_fund_backtest:
             v_asset_position, asset_trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
             return asset_trade_list
@@ -119,7 +119,7 @@ class FundEngine:
             v_fund_position, fund_trade_list, trigger_reason = self._trader.calc_trade(
                 dt, self._prep_target_fund_allocation, self._prep_target_asset_allocation,
                 cur_fund_position, cur_asset_position, 
-                self._prep_fund_nav, self._prep_fund_score, self._prep_asset_price
+                self._prep_fund_nav, self._prep_fund_score, self._prep_asset_price,self._prep_fund_unit_nav, is_real_trade
             )
             self._report_helper.update_rebalance_detail(dt, trigger_reason)
             return fund_trade_list
@@ -138,7 +138,7 @@ class FundEngine:
         cur_saa = self._saa_helper.on_price(dt, cur_asset_price)
         if self._taa_helper:
             asset_pct = self._dm.get_index_pcts_df(dt)
-            cur_taa = self._taa_helper.on_price(dt, cur_asset_price, cur_saa, asset_pct, {})
+            cur_taa = self._taa_helper.on_price(dt, cur_asset_price, cur_saa, asset_pct, {}, self._prep_fund_score)
         else:
             cur_taa = cur_saa
         return cur_taa
@@ -282,10 +282,12 @@ def fund_realtime(m: FundDataManager, saa: AssetWeight):
     calc_dt = dts.iloc[0]
     final_dt = dts.iloc[1]
     cash = b.DEFAULT_CASH_VALUE
+    #real asset position and fund position
     cur_asset_position = AssetPosition(cash=cash)
     cur_fund_position = FundPosition(cash=cash)
 
     b.prep_data(calc_dt)
+    #trade_list = b.calc_trade(calc_dt, cur_asset_position, cur_fund_position, True)
     trade_list = b.calc_trade(calc_dt, cur_asset_position, cur_fund_position)
     b.prep_data(final_dt)
     traded_list = b.finalize_trade(final_dt, trade_list, cur_asset_position, cur_fund_position)
