@@ -275,7 +275,7 @@ class FundPainter(object):
 
             plt.legend(fontsize=17, loc = 'lower left')
             plt.title(f'{asset} {b_d} {e_d}', fontsize=25)
-            #plt.suptitle(FundScoreManager().funcs[asset].__dict__, y=0.87, fontsize=18)
+            plt.suptitle(FundScoreManager().funcs[asset].__dict__, y=0.87, fontsize=18)
             ax.xaxis.set_ticks_position('top')
             fund_df = pd.DataFrame(res)
             fund_df = fund_df[['desc_name','fund_id','weight','alpha','beta','track_err','fee_rate','score_raw','score']]
