@@ -58,7 +58,8 @@ class ReportHelper:
         self.trade_df = pd.DataFrame(trade_list)
         if not self.trade_df.empty:
             self.trade_df['year'] = [_.year for _ in self.trade_df.trade_date]
-    
+        self.pct_df = dm._index_pct_df
+
     def setup(self, saa:AssetWeight):
         self.saa_weight = saa.__dict__
         self.asset_cash_history = {}
@@ -107,6 +108,7 @@ class ReportHelper:
     def _calc_stat_yearly(self, mv_df):
         if not self.trade_df.empty:
             trade_df = self.trade_df.set_index(['year','trade_date'])
+            trade_df = trade_df[trade_df['is_buy'] == True]
             trade_amount_year = trade_df.groupby('year').sum()
             mv_df = mv_df.reset_index()
             dates = mv_df['date'].values
@@ -124,10 +126,10 @@ class ReportHelper:
                     }
                 res.append(dic)
             self.turnover_df = pd.DataFrame(res)
-            self.turnover_df['turnover_rate_yearly'] = 100 * self.turnover_df['yearly_amount'] / self.turnover_df['year_begin_mv'] / 2
+            self.turnover_df['turnover_rate_yearly'] = 100 * self.turnover_df['yearly_amount'] / self.turnover_df['year_begin_mv']
             self.turnover_df['year_ret'] = self.turnover_df.year_end_mv / self.turnover_df.year_begin_mv - 1
             self.turnover_rate_yearly_avg = self.turnover_df['turnover_rate_yearly'].sum() / (mv_df.index[-1][0] - mv_df.index[0][0] + 1)
-            total_amount = self.trade_df.amount.sum()
+            total_amount = self.trade_df[self.trade_df['is_buy'] == True].amount.sum()
             self.turnover_rate_amt_mv_gmean = Calculator.get_turnover_rate(dates=dates, values=mv_df.mv.values, total_amount=total_amount)
         else:
             self.turnover_df = pd.DataFrame()
@@ -200,6 +202,15 @@ class ReportHelper:
         pos_df['desc_name'] = pos_df.index.map(lambda x : name_dict[x])
         pos_df.index.name = last_date
         pos_df = pos_df.sort_values('weight', ascending = False)
+        dt = sorted(list(self.fund_score_dm.keys()))[-1]
+        fund_rank = []
+        for i in pos_df.itertuples():
+            _score_dict = self.fund_score_dm[dt][i.index_id]
+            _score_list = sorted(_score_dict.items(), key=lambda x:x[1], reverse=True)
+            _fund_list = [i[0] for i in _score_list]
+            rank_i = _fund_list.index(i.Index) + 1 if i.Index in _fund_list else 10000
+            fund_rank.append(rank_i)
+        pos_df.loc[:,'fund_rank'] = fund_rank
         return pos_df
 
     def get_last_target_fund_allocation(self, target_fund):
@@ -242,6 +253,17 @@ class ReportHelper:
         fund_trade_df = pd.DataFrame(ft_res)
         return fund_trade_df
 
+    @staticmethod
+    def fund_rank_recent_analysis(dm, fund_id):
+        score_dict = dm._score_manager.score_cache
+        fund_info = dm.dts.fund_info
+        fund_indicator = dm.dts.fund_indicator
+        fund_nav = dm.dts.fund_nav
+        index_price = dm.dts.index_price
+        index_id = fund_info[fund_info['fund_id'] == fund_id].index_id.values[0]
+        score_func = dm._score_manager.funcs[index_id]
+        return HybridPainter.fund_rank_recent_analysis(fund_id,index_id,score_dict,fund_info,fund_indicator,fund_nav,index_price,score_func)
+
     def get_asset_trade(self):
         asset_trade = []
         for d in self.trade_history:
@@ -290,15 +312,24 @@ class ReportHelper:
         result = self._calc_stat(fund_mv).copy()
         _rb_df = pd.DataFrame(self._rebalance_details)
         submit_date_to_trade_day = trade_df[['submit_date','trade_date']].set_index('submit_date').to_dict()['trade_date']
-        turnover_df = trade_df.groupby('trade_date').sum()[['amount']].join(fund_mv).copy()
+        turnover_df = trade_df[trade_df['is_buy'] == True].groupby('submit_date').sum()[['amount']].join(fund_mv).copy().reset_index()
+        turnover_df['trade_date'] = turnover_df['submit_date'].map(lambda x : submit_date_to_trade_day[x])
+        turnover_df = turnover_df[['amount','mv','trade_date']].set_index('trade_date')
         _rb_df['trade_date'] = _rb_df['datetime'].map(lambda x : submit_date_to_trade_day[x])
         _rb_df['rebalance reason'] = _rb_df['rebalance_reason'].map(lambda x : x[0]['rebalanace_reason'])
-        _rb_df = _rb_df[['trade_date','rebalance reason']].set_index('trade_date').copy()
-        turnover_df['turnover'] = turnover_df['amount'] / turnover_df['mv'] / 2
+        _res = []
+        for dic in _rb_df['rebalance_reason']:
+            dic = dic[0]
+            strs = [ f'{v} ' for k , v in dic.items() if k is not 'rebalanace_reason']
+            strs = ''.join(strs)
+            _res.append(strs)
+        _rb_df['rebalance_detail'] = _res
+        _rb_df = _rb_df[['trade_date','rebalance reason','rebalance_detail']].set_index('trade_date').copy()
+        turnover_df['turnover'] = turnover_df['amount'] / turnover_df['mv']
         _df = pd.DataFrame([{'trade_date':d,'hold_year':l} for d,l in result['hold_years'].items()]).set_index('trade_date')
         _df.index = pd.to_datetime(_df.index)
         turnover_df = turnover_df.join(_df).join(_rb_df).copy()
-        return turnover_df[['turnover','hold_year','rebalance reason']]
+        return turnover_df[['turnover','hold_year','rebalance reason','rebalance_detail']]
 
     def backtest_asset_plot(self):
         bt_type = 'asset'
@@ -333,6 +364,12 @@ class ReportHelper:
         #     FundPainter.plot_fund_ret_each_year(self.turnover_df, w['mdd'], w['annual_ret'] )
         #     FundPainter.plot_turnover_rate_each_year(self.turnover_df, self.turnover_rate_yearly_avg)
 
+    def singel_index_plot(self):
+        w = self.get_fund_stat()
+        HybridPainter.plot_market_value_with_mdd_analysis(self.fund_mv, 'fund', self.index_price, self.saa_weight, w)
+        FundPainter.plot_fund_mdd_periods(self.fund_mv, self.fund_weights_history, self.fund_nav, self.fund_info)
+        FundPainter.plot_fund_alpha(self.fund_nav, self.fund_info, self.index_price, self.fund_position_history.copy(), w)
+
     def asset_price_ratio(self):    
         return HybridPainter.plot_asset_price_ratio(self.fund_nav, self.fund_info, self.index_price, self.fund_score_dm, self.get_fund_stat(), self.dts, self.saa_weight)
         
@@ -378,6 +415,14 @@ class ReportHelper:
         if bool(rebalance_reason):
             self._rebalance_details.append({'datetime':dt,'rebalance_reason':rebalance_reason})
 
+    def plot_taa_pct_df(self, dt):
+        res = []
+        for index_id, pct_data in TaaTunerParam.POOL.items():
+            res.append( self.pct_df.loc[index_id][['datetime',pct_data]].reset_index().pivot_table(index='datetime', columns='index_id')[pct_data])
+        pct_df = pd.concat(res, axis=1, sort=True)
+        df = pct_df.loc[dt:]
+        HybridPainter.plot_taa_detail(df)
+
     @staticmethod
     def get_taa_result(index_id:str, start_date:datetime.date, end_date:datetime.date, taaParam:TAAParam, dm:FundDataManager):
         taa_helper = TAAHelper(taa_params=taaParam)
@@ -386,7 +431,7 @@ class ReportHelper:
         fake_weight = AssetWeight(**{index_id:1})
         for dt in dts:
             asset_pct = dm.get_index_pcts_df(dt)
-            taa_helper.on_price(dt=dt, asset_price=None, cur_saa=fake_weight, asset_pct=asset_pct, select_val={})
+            taa_helper.on_price(dt=dt, asset_price=None, cur_saa=fake_weight, asset_pct=asset_pct, select_val={},score_dict={})
         df = pd.DataFrame(taa_helper.tactic_history).T.dropna()
         df['last_date'] = df.index.to_series().shift(1)
         con = df[index_id] != df[index_id].shift(1)
@@ -416,6 +461,11 @@ class ReportHelper:
                 res.append(dic)
         return pd.DataFrame(res).sort_values(['index_id','score'],ascending=[True,False]).set_index('index_id')
 
+    def get_mdd_stats(self):
+        fund_mv = pd.DataFrame([ {'date':k, 'mv':v} for k,v in self.fund_marktet_price_history.items()]).set_index('date')
+        res = Calculator.get_mdd_recover_result(fund_mv)
+        return res
+
     @staticmethod
     def plot_fund_score(fund_id:str='200002!0',begin_date:datetime.date=datetime.date(2015,1,1),dm:FundDataManager=None):
         FundPainter.plot_fund_rank(score_dic=dm._score_manager.score_cache,
@@ -493,21 +543,126 @@ class ReportHelper:
         return pd.concat(df_list)
 
     @staticmethod
-    def recent_week_return(mv_df,m,benchmark,begin_date=None,port_id=''):
-        today = datetime.date.today()
+    def recent_week_return(pos_df,fund_result,weight_df,saa,mv_df,m,benchmark,end_date=None,begin_date=None,port_id=''):
+        # 把买卖交易费平滑打入净值中
+        def _index_price_ratio_item(index_ratio_res, d1, d2, is_buy_fee, is_sell_fee):
+            _index_ratio_res = []
+            for index_id, fund_list in index_ratio_res.items():
+                _nav = m.dts.fund_nav[fund_list].loc[d1:d2]
+                if is_buy_fee:
+                    for fund_id in _nav:
+                        fund_i_fee = buy_dic.get(fund_id,0)
+                        fee_discount_l = np.linspace(1,1-fund_i_fee,_nav.shape[0])
+                        _nav[fund_id] = _nav[fund_id] * fee_discount_l
+                if is_sell_fee:
+                    for fund_id in _nav:
+                        fund_i_fee = sell_dic.get(fund_id,0)
+                        fee_discount_l = np.linspace(1,1-fund_i_fee,_nav.shape[0])
+                        _nav[fund_id] = _nav[fund_id] * fee_discount_l
+                _nav['index_price'] = m.dts.index_price.loc[d1:d2,index_id]
+                _nav = _nav/_nav.iloc[0]
+                _nav['mean'] = _nav.mean(axis=1)
+                _nav.loc[:,index_id] = _nav['mean'] / _nav['index_price']
+                _index_ratio_res.append(_nav[[index_id]].copy())   
+            res_df = pd.concat(_index_ratio_res, sort=True, axis=1)
+            return res_df
+
+        def _ret_item(begin_date, today, is_buy_fee, is_sell_fee):
+            _fund_list = list(weight_df[begin_date].keys())
+            _nav = m.dts.fund_nav[_fund_list].loc[begin_date:today]
+            fund_ret = _nav.iloc[-1] / _nav.iloc[0] - 1
+            ret_df = pd.DataFrame(fund_ret).rename(columns={0:'ret'})
+            res = []
+            for k,w in weight_df[begin_date].items():
+                res.append({'fund_id':k,'weight':w})
+            _weight_df = pd.DataFrame(res).set_index('fund_id')
+            res_df = ret_df.join(_weight_df)
+            res_df.loc[:,'rets'] = res_df['ret'] * res_df['weight']
+            res_df.loc[:,'begin_date'] = begin_date
+            res_df.loc[:,'end_date'] = today
+            return res_df
+
+        fund_info = m.dts.fund_info
+        buy_dic = (fund_info[['fund_id','purchase_fee']].set_index('fund_id') * 0.15).to_dict()['purchase_fee']
+        sell_dic = fund_info[['fund_id','redeem_fee']].set_index('fund_id').to_dict()['redeem_fee']
+        index_price = m.dts.index_price
+        if end_date is None:
+            today = datetime.date.today()
+        else:
+            today = end_date
         if begin_date is None:
             begin_date =  today - datetime.timedelta(days=today.weekday())
         trading_days = sorted(list(set(m.dts.trading_days.datetime) & set(mv_df.index)))
         mv_df = mv_df.loc[begin_date:].reindex(trading_days)
-        stats = Calculator.get_stat_result_from_df(df=mv_df.reset_index(), date_column='date', value_column='mv')
+        stats = Calculator.get_stat_result_from_df(df=mv_df.dropna().reset_index(), date_column='date', value_column='mv')
         annual_ret= round(stats.annualized_ret,3)
         mdd = round(stats.mdd,4)
-        df_plot_i = mv_df.join(m.dts.index_price[[benchmark]]).loc[begin_date:]
+        df_plot_i = mv_df.join(m.dts.index_price[[benchmark]]).loc[begin_date:today]
         df_plot_i = df_plot_i / df_plot_i.iloc[0]
-        #df_plot_i.plot.line(figsize=(12,6))
-        total_ret = round((stats.last_unit_nav - 1) * 10000, 1)
+        total_ret = int((stats.last_unit_nav - 1) * 10000)
         st = f'check period annual ret : {annual_ret} mdd : {mdd} total_ret {total_ret} bp'
         FundPainter.plot_recent_ret(df_plot_i,begin_date,today,st,port_id)
+        _index_list = []
+        for asset, w in saa.__dict__.items():
+            if w > 0 and asset != 'cash':
+                _index_list.append(asset)
+        _index_price = index_price.loc[begin_date:today]
+        _index_price = _index_price / _index_price.iloc[0]
+        AssetPainter.plot_index_price_ratio(_index_price,'index_price')
+        
+        if begin_date >= fund_result['rebalance_date'][-1]:
+            res_df = _ret_item(begin_date,today, False, False)
+        else:
+            date_list = [[begin_date,fund_result['rebalance_date'][-1]],[fund_result['rebalance_date'][-1],today]]
+            _res_dt_df = []
+            for idx,dl in enumerate(date_list):
+                b1 = dl[0]
+                b2 = dl[1]
+                _res_dt_df.append(_ret_item(b1, b2, False, False))
+            res_df = pd.concat(_res_dt_df, axis=0, sort=True)
+        #res_df.loc[:,'rets'] = res_df['ret'] * res_df['weight']
+        res_df = res_df.sort_values('rets',ascending=False)
+        res_df.loc[:,'rets'] = [str(int(i*10000))+'BP' for i in res_df.loc[:,'rets']]
+        desc_dic = m.dts.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
+        index_dic = m.dts.fund_info[['fund_id','index_id']].set_index('fund_id').to_dict()['index_id']
+        res_df['desc_name'] = res_df.index.map(lambda x : desc_dic[x])
+        res_df['index_id'] = res_df.index.map(lambda x : index_dic[x])        
+        if begin_date > fund_result['rebalance_date'][-1]:
+            d1 = begin_date
+            d2 = today
+            index_ratio_res = {}
+            _fund_list = list(weight_df[begin_date].keys())
+            for fund_id in _fund_list:
+                _index_id = index_dic[fund_id]
+                if _index_id not in index_ratio_res:
+                    index_ratio_res[_index_id] = [fund_id]
+                else:
+                    index_ratio_res[_index_id].append(fund_id)
+            res_index_df = _index_price_ratio_item(index_ratio_res, d1, d2, False, False)
+        else:
+            date_list = [[begin_date,fund_result['rebalance_date'][-1]],[fund_result['rebalance_date'][-1],today]]
+            _res_df = []
+            for idx,dl in enumerate(date_list):
+                b1 = dl[0]
+                b2 = dl[1]
+                _res = {}
+                for fund_id, fund_pos_i in pos_df[b1].items():
+                    if fund_pos_i['volume'] > 1:
+                        if fund_pos_i['index_id'] not in _res:
+                            _res[fund_pos_i['index_id']] = [fund_pos_i['fund_id']]
+                        else:
+                            _res[fund_pos_i['index_id']].append(fund_pos_i['fund_id'])
+                if idx == 0:
+                    _df = _index_price_ratio_item(_res, b1, b2, False, True)
+                elif idx == 1:
+                    _df = _index_price_ratio_item(_res, b1, b2, True, False)
+                _res_df.append(_df)
+            df_1 = _res_df[0]
+            df_2 = _res_df[1]
+            df_2 = df_1.iloc[-1] * df_2
+            res_index_df = df_1.iloc[:-1].append(df_2, sort=True)
+        AssetPainter.plot_index_price_ratio(res_index_df,'price_ratio')
+        return res_df[['begin_date','end_date','desc_name','index_id','weight','rets']]
 
     @staticmethod
     def rolling_analysis(rolling_year, index_id, m, mv_df, fund_annual_ret):
@@ -526,4 +681,35 @@ class ReportHelper:
         index_price = m.dts.index_price
         fund_nav = m.dts.fund_nav
         result_df, pos_df = HybridPainter.regular_rebalance_in_index_strategy(begin_date,end_date,index_id,rebalance_year,top_funds,dts,fund_score_dic,fund_info,index_price,fund_nav)
-        return result_df, pos_df
\ No newline at end of file
+        return result_df, pos_df
+
+    @staticmethod
+    def mining_fac_importances(m,start_date,end_date,index_id,top_num,wind_class_2):
+        '''
+        index_id = 'csi500'
+        start_date = datetime.date(2020,1,1)
+        end_date = datetime.date(2020,8,10)
+        top_num = 5
+        wind_class_2 = ['被动指数型基金']
+        ReportHelper.mining_fac_importances(m,start_date,end_date,index_id,top_num,wind_class_2)
+        '''
+        _dts = m.dts.trading_days.datetime
+        _dts = _dts[(_dts >= start_date) & (_dts <= end_date)]
+        _start_date = _dts.iloc[0]
+        _end_date = _dts.iloc[-1]
+        fund_info = m.dts.fund_info
+        _fund_list = fund_info[(fund_info['index_id'] == index_id) & fund_info['wind_class_2'].isin(wind_class_2)].fund_id.tolist()
+        _index_fund_pack = m.dts.index_fund_indicator_pack.loc[index_id].loc[_start_date:_end_date]
+        _dts = _index_fund_pack.index.get_level_values(0).tolist()
+        _col1 = m.dts.fund_nav.columns.tolist()
+        fl = [i for i in _fund_list if i in _col1]
+        _fund_nav = m.dts.fund_nav[fl].loc[_start_date:_end_date].dropna(axis=1)
+        top_funds = (_fund_nav.iloc[-1] / _fund_nav.iloc[0]).sort_values(ascending=False).index.tolist()
+        indicator_existed_fund = _index_fund_pack.loc[_start_date].index.tolist()
+        top_funds = [i for i in top_funds if i in indicator_existed_fund][:top_num]
+        cur_d = _index_fund_pack.loc[_start_date]
+        cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+        cur_d_sta['beta'] = cur_d['beta']
+        cur_d_sta['year_length'] = cur_d['year_length']
+        cur_d = cur_d_sta
+        return pd.DataFrame(cur_d.rank(pct=True).loc[top_funds].mean(axis=0)).rename(columns={0:'rank'}).sort_values('rank',ascending=False)
\ No newline at end of file
