@@ -67,8 +67,8 @@ class AssetTrader(Helper):
                 new_position.update(trd)
             #如果一个交易单过小，不达到交易下限，会被忽略，导致买单和卖单交易额相加不相等
             buy_trades_amt = sum([i.amount for i in trades if i.is_buy is True])
-            sell_trades_amt = sum([i.mark_price * i.volume for i in trades if i.is_buy is False]) 
-            buy_adjusted = sell_trades_amt / buy_trades_amt if sell_trades_amt > 0 else 1
+            sell_trades_amt = sum([i.mark_price * i.volume for i in trades if i.is_buy is False])
+            buy_adjusted = sell_trades_amt / buy_trades_amt if (sell_trades_amt > 0 and buy_trades_amt > 0) else 1
             for trd in trades:
                 if trd.is_buy == True:
                     trd.amount *= buy_adjusted
@@ -105,7 +105,7 @@ class AssetTrader(Helper):
         # 保证卖单和买单实际成交的金额一致，就不会出现现金比例波动的情况，现金就可以设置为0，去掉现金资产比例 
         plan_sel_amount = 0 if len(_trade_list_sel) < 1 else sum([i.amount for i in _trade_list_sel])
         plan_buy_amount = 0 if len(_trade_list_buy) < 1 else sum([i.amount for i in _trade_list_buy])
-        if plan_sel_amount == 0: 
+        if plan_sel_amount == 0 or plan_buy_amount == 0: 
             buy_amount_adjusted_rate = 1
         else:
             buy_amount_adjusted_rate = plan_sel_amount / plan_buy_amount
@@ -246,7 +246,7 @@ class FundTrader(AssetTrader):
         _trade_list_sel = [i  for i in traded_list if i.is_buy is False]
         plan_sel_amount = 0 if len(_trade_list_sel) < 1 else sum([i.amount for i in _trade_list_sel])
         plan_buy_amount = 0 if len(_trade_list_buy) < 1 else sum([i.amount for i in _trade_list_buy])
-        if plan_sel_amount == 0: 
+        if plan_sel_amount == 0 or plan_buy_amount == 0: 
             buy_amount_adjusted_rate = 1
         else:
             buy_amount_adjusted_rate = plan_sel_amount / plan_buy_amount
@@ -262,6 +262,7 @@ class FundTrader(AssetTrader):
                     purchase_fee = 0
                 trd.volume = trd.amount / trd.trade_price / (1 + purchase_fee)
                 trd.commission = trd.volume * trd.trade_price * purchase_fee
+                trd.fund_unit_volume = trd.amount / trd.fund_unit_nav
             else:
                 trd.commission = 0            
                 #print(f'trade is not permitted : {trd}')
@@ -328,7 +329,6 @@ class FundTrader(AssetTrader):
                 index_rebalance_max_diff = abs(c.index_diff)
                 _index_rebalance_max_diff_name = index_id
 
-            # calc funds in each index
             c.fund_cur_wgts = cur_fund_position.calc_mv_n_w(cur_fund_nav, index_id)[1]
             c.cur_fund_ids = set(c.fund_cur_wgts.keys())
             c.fund_scores = sorted(cur_fund_score.get(index_id, {}).items(), key=lambda item: item[1], reverse=True)
@@ -415,7 +415,9 @@ class FundTrader(AssetTrader):
             cur_asset_position: AssetPosition,
             cur_fund_nav: dict,
             cur_fund_score: dict,
-            cur_asset_price: AssetPrice) -> list:
+            cur_asset_price: AssetPrice,
+            cur_fund_unit_nav: dict,
+            is_real_trade: bool) -> list:
         '''
         v_asset_position, asset_trade_list = self.calc_asset_trade(dt, cur_asset_position, cur_asset_price, tar_asset_weight)
         if not asset_trade_list:
@@ -425,20 +427,27 @@ class FundTrader(AssetTrader):
         '''
         index_fund_cache = []
 
-        trigger, trigger_reason = self.judge_trade(dt, index_fund_cache, 
-                                    tar_fund_weight, tar_asset_weight,
-                                    cur_fund_position, cur_asset_position, cur_fund_nav, 
-                                    cur_fund_score, cur_asset_price)
+        if is_real_trade:
+            for fund_id, pos_info in cur_fund_position.funds.items():
+                pos_info.volume = pos_info.unit_volume
+            trigger, trigger_reason = self.judge_trade(dt, index_fund_cache, 
+                                        tar_fund_weight, tar_asset_weight,
+                                        cur_fund_position, cur_asset_position, cur_fund_unit_nav, 
+                                        cur_fund_score, cur_asset_price)
+        else:
+            trigger, trigger_reason = self.judge_trade(dt, index_fund_cache, 
+                                        tar_fund_weight, tar_asset_weight,
+                                        cur_fund_position, cur_asset_position, cur_fund_nav, 
+                                        cur_fund_score, cur_asset_price)
         
-        if trigger == 0:
+        # 如果是真实交易 且cash 不为零，按照当日cash，返回买单
+        if (is_real_trade is False and trigger == 0) or (is_real_trade is True and cur_fund_position.cash == 0 and trigger == 0):
             return cur_fund_position, [], {}
 
         fund_trades = self.get_trades(dt, index_fund_cache, 
                                     tar_fund_weight, tar_asset_weight,
                                     cur_fund_position, cur_asset_position, cur_fund_nav, 
-                                    cur_fund_score, cur_asset_price)
-
-
+                                    cur_fund_score, cur_asset_price, cur_fund_unit_nav, is_real_trade)
         fund_trades.sort(key=lambda x: x.is_buy)
         new_fund_position = cur_fund_position.copy()
 
@@ -447,7 +456,6 @@ class FundTrader(AssetTrader):
             new_fund_position.update(_trade)
         return new_fund_position, fund_trades, trigger_reason
 
-
     def get_trades(self, dt,
             index_fund_cache, 
             tar_fund_weight: FundWeight,
@@ -456,12 +464,15 @@ class FundTrader(AssetTrader):
             cur_asset_position: AssetPosition,
             cur_fund_nav: dict,
             cur_fund_score: dict,
-            cur_asset_price: AssetPrice):
-        
-        fund_tot_mv, cur_fund_wgts = cur_fund_position.calc_mv_n_w(fund_navs=cur_fund_nav)
-        
+            cur_asset_price: AssetPrice,
+            cur_fund_unit_nav: dict,
+            is_real_trade: bool):
+        # 如果是真实交易， 把单位份额赋值给复权份额， 价格不用后复权，用真实单位净值
+        if is_real_trade:
+            fund_tot_mv, cur_fund_wgts = cur_fund_position.calc_mv_n_w(fund_navs=cur_fund_unit_nav)
+        else:
+            fund_tot_mv, cur_fund_wgts = cur_fund_position.calc_mv_n_w(fund_navs=cur_fund_nav)
         fund_trades = []
-
         for c in index_fund_cache:
             index_id = c.index_id
             tar_fund_list = tar_fund_weight.get_funds(index_id)
@@ -469,7 +480,7 @@ class FundTrader(AssetTrader):
             for fund_id in set(tar_fund_list).union(cur_fund_list):
                 target_fund_amt = tar_fund_weight.get_wgt(fund_id) * fund_tot_mv
                 cur_fund_volume = cur_fund_position.get_volume(fund_id) or 0
-                p = cur_fund_nav[fund_id]
+                p = cur_fund_unit_nav[fund_id] if is_real_trade else cur_fund_nav[fund_id]
                 cur_fund_amt = cur_fund_volume * p
 
                 if abs(target_fund_amt - cur_fund_amt) > fund_tot_mv * self.fund_param.MinCountAmtDiff \
@@ -477,14 +488,17 @@ class FundTrader(AssetTrader):
                     # 如果是清某一只基金的逻辑，清空可以执行
                     # commision 放到 finalize_trade 部分执行
                     is_buy = target_fund_amt > cur_fund_amt
+                    _amt = abs(target_fund_amt - cur_fund_amt)
                     if is_buy:
                         _trade = FundTrade(
                             fund_id=fund_id,
                             index_id=index_id,
                             mark_price=p,
-                            amount=abs(target_fund_amt - cur_fund_amt),
+                            amount=_amt,
                             is_buy=is_buy,
-                            submit_date=dt
+                            submit_date=dt,
+                            fund_unit_nav=cur_fund_unit_nav[fund_id],
+                            fund_unit_volume=_amt/cur_fund_unit_nav[fund_id]
                         )
                     else:
                         is_to_clean_up = target_fund_amt == 0 and cur_fund_amt > 0
@@ -496,19 +510,34 @@ class FundTrader(AssetTrader):
                             volume= cur_fund_volume if is_to_clean_up else abs(target_fund_amt - cur_fund_amt)/p,
                             is_buy=is_buy,
                             submit_date=dt,
-                            is_to_cleanup=is_to_clean_up
+                            is_to_cleanup=is_to_clean_up,
+                            fund_unit_nav=cur_fund_unit_nav[fund_id],
+                            fund_unit_volume=_amt/cur_fund_unit_nav[fund_id],
                         )
                     fund_trades.append(_trade)
                     # print(f'(fund){fund_id} (d){"buy" if is_buy else "sell"} (r){c.fund_ranks.get(fund_id, -1)} (p){p} (amt){target_fund_amt - cur_fund_amt} (tar){target_fund_amt} (ind){index_id}')
         
-        #如果一个交易单过小，不达到交易下限，会被忽略，导致买单和卖单交易额相加不相等
-        buy_trades_amt = sum([i.amount for i in fund_trades if i.is_buy is True])
-        sell_trades_amt = sum([i.mark_price * i.volume for i in fund_trades if i.is_buy is False]) 
-        buy_adjusted = sell_trades_amt / buy_trades_amt if sell_trades_amt > 0 else 1
-        for trd in fund_trades:
-            if trd.is_buy == True:
-                trd.amount *= buy_adjusted
-        
+        if is_real_trade and cur_fund_position.cash > 0 :
+            #如果是真实交易，现金不为0， 按照cash值调整买单交易量，只返回买单
+            buy_trades_amt = sum([i.amount for i in fund_trades if i.is_buy is True])
+            sell_trades_amt = cur_fund_position.cash
+            buy_adjusted = sell_trades_amt / buy_trades_amt if (sell_trades_amt > 0 and buy_trades_amt > 0) else 1
+            _fund_trades = []
+            for trd in fund_trades:
+                if trd.is_buy == True:
+                    trd.amount *= buy_adjusted
+                    trd.fund_unit_volume = trd.amount/trd.fund_unit_nav
+                    _fund_trades.append(trd)
+            fund_trades = _fund_trades
+        else:
+            #如果一个交易单过小，不达到交易下限，会被忽略，导致买单和卖单交易额相加不相等
+            buy_trades_amt = sum([i.amount for i in fund_trades if i.is_buy is True])
+            sell_trades_amt = sum([i.mark_price * i.volume for i in fund_trades if i.is_buy is False]) 
+            buy_adjusted = sell_trades_amt / buy_trades_amt if (sell_trades_amt > 0 and buy_trades_amt > 0) else 1
+            for trd in fund_trades:
+                if trd.is_buy == True:
+                    trd.amount *= buy_adjusted
+                    trd.fund_unit_volume = trd.amount/trd.fund_unit_nav
         ## 检查买卖量是否一致
         # buy_trades_amt = sum([i.amount for i in fund_trades if i.is_buy is True])
         # sell_trades_amt = sum([i.mark_price * i.volume for i in fund_trades if i.is_buy is False]) 
