@@ -180,7 +180,7 @@ class Calculator:
                           values: pandas.core.series.Series,
                           total_amount: float):
         year_num = len(list(set([_.year for _ in dates])))
-        return total_amount / gmean(values) / 2 / year_num * 100
+        return total_amount / gmean(values) / year_num * 100
 
     @staticmethod
     def get_stat_result_from_df(df: pandas.DataFrame,
@@ -274,7 +274,7 @@ class Calculator:
         taa_helper = TAAHelper(taa_params=taa_param)
         for dt in dts:
             asset_pct = index_val_data.loc[dt]
-            taa_helper.on_price(dt=dt, asset_price=None, cur_saa=fake_weight, asset_pct=asset_pct, select_val={asset_func_index:index_val})
+            taa_helper.on_price(dt=dt, asset_price=None, cur_saa=fake_weight, asset_pct=asset_pct, select_val={asset_func_index:index_val}, score_dict={})
         df = pandas.DataFrame(taa_helper.tactic_history).T.dropna()
         df['last_date'] = df.index.to_series().shift(1)
         con = df[asset_func_index] != df[asset_func_index].shift(1)
@@ -366,6 +366,67 @@ class Calculator:
         res = [i for i in res if i['val_level'] != 'median']
         return res
 
+    @staticmethod
+    def get_mdd_recover_result(mv_df:pandas.DataFrame):
+
+        def _get_mdd_recover_result_df(df:pandas.DataFrame, date_column:str, value_column:str):
+            dates = df[date_column].values
+            values = df[value_column].values
+            return _mdd_recover_analysis(values,dates)
+
+        def _mdd_recover_analysis(values,dates):
+            sr = pandas.Series(values, index=dates).sort_index()
+            if sr.empty:
+                mdd = 0
+                mdd_date1 = None
+                mdd_date2 = None
+                mdd_lens = 0
+                return mdd, mdd_date1, mdd_date2, mdd_lens
+            mdd_part =  sr[:] / sr[:].rolling(window=sr.shape[0], min_periods=1).max()
+            mdd = 1 - mdd_part.min()
+            if mdd == 0:
+                mdd_date1 = None
+                mdd_date2 = None
+                mdd_lens = 0
+            else:
+                mdd_date = mdd_part.idxmin()
+                mdd_date1 = sr[:mdd_date].idxmax()
+                sr_tmp = sr[mdd_date1:]
+                recover_sr = sr_tmp[sr_tmp> sr[mdd_date1]]
+                if recover_sr.empty:
+                    mdd_date2 = sr_tmp.index[-1]
+                else: 
+                    mdd_date2 = sr_tmp[sr_tmp> sr[mdd_date1]].index[0]
+                mdd_lens = sr.loc[mdd_date1:mdd_date2].shape[0]
+            return mdd, mdd_date1, mdd_date2, mdd_lens
+
+        mdd1, mdd_date1_1, mdd_date1_2, mdd_lens1 = _get_mdd_recover_result_df(mv_df.reset_index(),'date','mv')
+        mv_df_1 = mv_df.loc[:mdd_date1_1].reset_index()
+        mv_df_2 = mv_df.loc[mdd_date1_2:].reset_index()
+
+        mdd2, mdd_date2_1, mdd_date2_2, mdd_lens2 = _get_mdd_recover_result_df(mv_df_1,'date','mv')
+        mdd3, mdd_date3_1, mdd_date3_2, mdd_lens3 = _get_mdd_recover_result_df(mv_df_2,'date','mv')
+        if mdd3 > mdd2:
+            mdd2 = mdd3
+            mdd_date2_1 = mdd_date3_1
+            mdd_date2_2 = mdd_date3_2
+            mdd_lens2 = mdd_lens3
+
+        res = {
+            'mdd1':mdd1,
+            'mdd1_d1':mdd_date1_1,
+            'mdd1_d2':mdd_date1_2,
+            'mdd1_lens':mdd_lens1,
+            'mdd2':mdd2,
+            'mdd2_d1':mdd_date2_1,
+            'mdd2_d2':mdd_date2_2,
+            'mdd2_len':mdd_lens2
+        }
+
+        return res
+
+
+
 if __name__ == "__main__":
     # get pct with select rank
     Calculator.get_index_pct_data_df()
